# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/proto/function.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/proto/function.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from zetasql.wasi._pb2.zetasql.public import annotation_pb2 as zetasql_dot_public_dot_annotation__pb2
from zetasql.wasi._pb2.zetasql.public import constness_level_pb2 as zetasql_dot_public_dot_constness__level__pb2
from zetasql.wasi._pb2.zetasql.public import deprecation_warning_pb2 as zetasql_dot_public_dot_deprecation__warning__pb2
from zetasql.wasi._pb2.zetasql.public import function_pb2 as zetasql_dot_public_dot_function__pb2
from zetasql.wasi._pb2.zetasql.public import options_pb2 as zetasql_dot_public_dot_options__pb2
from zetasql.wasi._pb2.zetasql.public import parse_location_range_pb2 as zetasql_dot_public_dot_parse__location__range__pb2
from zetasql.wasi._pb2.zetasql.public import parse_resume_location_pb2 as zetasql_dot_public_dot_parse__resume__location__pb2
from zetasql.wasi._pb2.zetasql.public import simple_table_pb2 as zetasql_dot_public_dot_simple__table__pb2
from zetasql.wasi._pb2.zetasql.public import type_pb2 as zetasql_dot_public_dot_type__pb2
from zetasql.wasi._pb2.zetasql.public import value_pb2 as zetasql_dot_public_dot_value__pb2
from zetasql.wasi._pb2.zetasql.resolved_ast import resolved_ast_enums_pb2 as zetasql_dot_resolved__ast_dot_resolved__ast__enums__pb2
from zetasql.wasi._pb2.zetasql.resolved_ast import serialization_pb2 as zetasql_dot_resolved__ast_dot_serialization__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1czetasql/proto/function.proto\x12\x07zetasql\x1a\x1fzetasql/public/annotation.proto\x1a$zetasql/public/constness_level.proto\x1a(zetasql/public/deprecation_warning.proto\x1a\x1dzetasql/public/function.proto\x1a\x1czetasql/public/options.proto\x1a)zetasql/public/parse_location_range.proto\x1a*zetasql/public/parse_resume_location.proto\x1a!zetasql/public/simple_table.proto\x1a\x19zetasql/public/type.proto\x1a\x1azetasql/public/value.proto\x1a-zetasql/resolved_ast/resolved_ast_enums.proto\x1a(zetasql/resolved_ast/serialization.proto\"\xa1\x02\n\x16TVFRelationColumnProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12 \n\x04type\x18\x02 \x01(\x0b\x32\x12.zetasql.TypeProto\x12\x18\n\x10is_pseudo_column\x18\x03 \x01(\x08\x12\x33\n\x0e\x61nnotation_map\x18\x06 \x01(\x0b\x32\x1b.zetasql.AnnotationMapProto\x12\x43\n\x19name_parse_location_range\x18\x04 \x01(\x0b\x32 .zetasql.ParseLocationRangeProto\x12\x43\n\x19type_parse_location_range\x18\x05 \x01(\x0b\x32 .zetasql.ParseLocationRangeProto\"b\n\x10TVFRelationProto\x12/\n\x06\x63olumn\x18\x01 \x03(\x0b\x32\x1f.zetasql.TVFRelationColumnProto\x12\x1d\n\x0eis_value_table\x18\x02 \x01(\x08:\x05\x66\x61lse\"0\n\rTVFModelProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tfull_name\x18\x02 \x01(\t\"5\n\x12TVFConnectionProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tfull_name\x18\x02 \x01(\t\")\n\x12TVFDescriptorProto\x12\x13\n\x0b\x63olumn_name\x18\x01 \x03(\t\"0\n\rTVFGraphProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tfull_name\x18\x02 \x01(\t\"\xfe\n\n FunctionArgumentTypeOptionsProto\x12?\n\x0b\x63\x61rdinality\x18\x01 \x01(\x0e\x32*.zetasql.FunctionEnums.ArgumentCardinality\x12\x1c\n\x10must_be_constant\x18\x02 \x01(\x08\x42\x02\x18\x01\x12\x18\n\x10must_be_non_null\x18\x03 \x01(\x08\x12\x18\n\x10is_not_aggregate\x18\x04 \x01(\x08\x12\x1d\n\x15must_support_equality\x18\x05 \x01(\x08\x12\x1d\n\x15must_support_ordering\x18\x06 \x01(\x08\x12\x11\n\tmin_value\x18\x07 \x01(\x03\x12\x11\n\tmax_value\x18\x08 \x01(\x03\x12,\n$extra_relation_input_columns_allowed\x18\t \x01(\x08\x12\x38\n\x15relation_input_schema\x18\n \x01(\x0b\x32\x19.zetasql.TVFRelationProto\x12\x15\n\rargument_name\x18\x0b \x01(\t\x12\x46\n\x1c\x61rgument_name_parse_location\x18\x0c \x01(\x0b\x32 .zetasql.ParseLocationRangeProto\x12\x46\n\x1c\x61rgument_type_parse_location\x18\r \x01(\x0b\x32 .zetasql.ParseLocationRangeProto\x12M\n\x17procedure_argument_mode\x18\x0e \x01(\x0e\x32,.zetasql.FunctionEnums.ProcedureArgumentMode\x12-\n\x1a\x61rgument_name_is_mandatory\x18\x0f \x01(\x08:\x05\x66\x61lseB\x02\x18\x01\x12.\n\"descriptor_resolution_table_offset\x18\x10 \x01(\x05:\x02-1\x12*\n\rdefault_value\x18\x11 \x01(\x0b\x32\x13.zetasql.ValueProto\x12.\n\x12\x64\x65\x66\x61ult_value_type\x18\x12 \x01(\x0b\x32\x12.zetasql.TypeProto\x12p\n\x17\x61rgument_collation_mode\x18\x13 \x01(\x0e\x32,.zetasql.FunctionEnums.ArgumentCollationMode:!AFFECTS_OPERATION_AND_PROPAGATION\x12(\n uses_array_element_for_collation\x18\x14 \x01(\x08\x12\x1d\n\x15must_support_grouping\x18\x15 \x01(\x08\x12+\n#array_element_must_support_equality\x18\x16 \x01(\x08\x12+\n#array_element_must_support_ordering\x18\x17 \x01(\x08\x12+\n#array_element_must_support_grouping\x18\x18 \x01(\x08\x12\x45\n\x13named_argument_kind\x18\x19 \x01(\x0e\x32(.zetasql.FunctionEnums.NamedArgumentKind\x12[\n\x13\x61rgument_alias_kind\x18\x1a \x01(\x0e\x32(.zetasql.FunctionEnums.ArgumentAliasKind:\x14\x41RGUMENT_NON_ALIASED\x12\'\n\x1bmust_be_constant_expression\x18\x1b \x01(\x08\x42\x02\x18\x01\x12;\n\x0f\x63onstness_level\x18\x1c \x01(\x0e\x32\".zetasql.ConstnessLevelProto.Level\"\x81\x01\n\x17\x41rgumentTypeLambdaProto\x12\x34\n\x08\x61rgument\x18\x05 \x03(\x0b\x32\".zetasql.FunctionArgumentTypeProto\x12\x30\n\x04\x62ody\x18\x06 \x01(\x0b\x32\".zetasql.FunctionArgumentTypeProto\"\xf2\x01\n\x19\x46unctionArgumentTypeProto\x12,\n\x04kind\x18\x01 \x01(\x0e\x32\x1e.zetasql.SignatureArgumentKind\x12 \n\x04type\x18\x02 \x01(\x0b\x32\x12.zetasql.TypeProto\x12\x17\n\x0fnum_occurrences\x18\x04 \x01(\x05\x12:\n\x07options\x18\x03 \x01(\x0b\x32).zetasql.FunctionArgumentTypeOptionsProto\x12\x30\n\x06lambda\x18\x05 \x01(\x0b\x32 .zetasql.ArgumentTypeLambdaProto\"\xb4\x01\n$FunctionSignatureRewriteOptionsProto\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12-\n\x08rewriter\x18\x02 \x01(\x0e\x32\x1b.zetasql.ResolvedASTRewrite\x12\x0b\n\x03sql\x18\x03 \x01(\t\x12\x1e\n\x16\x61llow_table_references\x18\x04 \x01(\x08\x12\x1f\n\x17\x61llowed_function_groups\x18\x05 \x03(\t\"\xa7\x03\n\x1d\x46unctionSignatureOptionsProto\x12\x1c\n\ris_deprecated\x18\x02 \x01(\x08:\x05\x66\x61lse\x12O\n\x1e\x61\x64\x64itional_deprecation_warning\x18\x03 \x03(\x0b\x32\'.zetasql.FreestandingDeprecationWarning\x12;\n\x19required_language_feature\x18\x04 \x03(\x0e\x32\x18.zetasql.LanguageFeature\x12#\n\x14is_aliased_signature\x18\x05 \x01(\x08:\x05\x66\x61lse\x12\"\n\x14propagates_collation\x18\x06 \x01(\x08:\x04true\x12\'\n\x18uses_operation_collation\x18\x07 \x01(\x08:\x05\x66\x61lse\x12 \n\x11rejects_collation\x18\x08 \x01(\x08:\x05\x66\x61lse\x12\x46\n\x0frewrite_options\x18\t \x01(\x0b\x32-.zetasql.FunctionSignatureRewriteOptionsProto\"\xd4\x01\n\x16\x46unctionSignatureProto\x12\x34\n\x08\x61rgument\x18\x01 \x03(\x0b\x32\".zetasql.FunctionArgumentTypeProto\x12\x37\n\x0breturn_type\x18\x02 \x01(\x0b\x32\".zetasql.FunctionArgumentTypeProto\x12\x12\n\ncontext_id\x18\x03 \x01(\x03\x12\x37\n\x07options\x18\x04 \x01(\x0b\x32&.zetasql.FunctionSignatureOptionsProto\"\xb6\x06\n\x14\x46unctionOptionsProto\x12#\n\x14supports_over_clause\x18\x01 \x01(\x08:\x05\x66\x61lse\x12]\n\x17window_ordering_support\x18\x02 \x01(\x0e\x32).zetasql.FunctionEnums.WindowOrderSupport:\x11ORDER_UNSUPPORTED\x12&\n\x17supports_window_framing\x18\x03 \x01(\x08:\x05\x66\x61lse\x12%\n\x17\x61rguments_are_coercible\x18\x04 \x01(\x08:\x04true\x12\x1c\n\ris_deprecated\x18\x05 \x01(\x08:\x05\x66\x61lse\x12\x12\n\nalias_name\x18\x06 \x01(\t\x12\x10\n\x08sql_name\x18\x07 \x01(\t\x12\"\n\x14\x61llow_external_usage\x18\x08 \x01(\x08:\x04true\x12@\n\nvolatility\x18\t \x01(\x0e\x32!.zetasql.FunctionEnums.Volatility:\tIMMUTABLE\x12 \n\x11supports_order_by\x18\n \x01(\x08:\x05\x66\x61lse\x12;\n\x19required_language_feature\x18\x0b \x03(\x0e\x32\x18.zetasql.LanguageFeature\x12\x1d\n\x0esupports_limit\x18\x0c \x01(\x08:\x05\x66\x61lse\x12.\n\x1fsupports_null_handling_modifier\x18\r \x01(\x08:\x05\x66\x61lse\x12&\n\x18supports_safe_error_mode\x18\x0e \x01(\x08:\x04true\x12&\n\x18supports_having_modifier\x18\x0f \x01(\x08:\x04true\x12\x30\n!supports_clamped_between_modifier\x18\x10 \x01(\x08:\x05\x66\x61lse\x12&\n\x18uses_upper_case_sql_name\x18\x11 \x01(\x08:\x04true\x12(\n\x19may_suppress_side_effects\x18\x12 \x01(\x08:\x05\x66\x61lse\x12\x1f\n\x17module_name_from_import\x18\x13 \x03(\t\"\xda\x03\n\rFunctionProto\x12\x11\n\tname_path\x18\x01 \x03(\t\x12\r\n\x05group\x18\x02 \x01(\t\x12)\n\x04mode\x18\x03 \x01(\x0e\x32\x1b.zetasql.FunctionEnums.Mode\x12\x32\n\tsignature\x18\x04 \x03(\x0b\x32\x1f.zetasql.FunctionSignatureProto\x12.\n\x07options\x18\x05 \x01(\x0b\x32\x1d.zetasql.FunctionOptionsProto\x12@\n\x15parse_resume_location\x18\x08 \x01(\x0b\x32!.zetasql.ParseResumeLocationProto\x12,\n$templated_sql_function_argument_name\x18\x07 \x03(\t\x12\x61\n\x0csql_security\x18\t \x01(\x0e\x32\x31.zetasql.ResolvedCreateStatementEnums.SqlSecurity:\x18SQL_SECURITY_UNSPECIFIED\x12\x45\n\x11statement_context\x18\n \x01(\x0e\x32\x19.zetasql.StatementContext:\x0f\x43ONTEXT_DEFAULT\"\x1f\n\x1dResolvedFunctionCallInfoProto\"\x86\x01\n\x1fTableValuedFunctionOptionsProto\x12&\n\x18uses_upper_case_sql_name\x18\x01 \x01(\x08:\x04true\x12;\n\x19required_language_feature\x18\x02 \x03(\x0e\x32\x18.zetasql.LanguageFeature\"\xd4\x04\n\x18TableValuedFunctionProto\x12\x11\n\tname_path\x18\x01 \x03(\t\x12\x36\n\tsignature\x18\x02 \x01(\x0b\x32\x1f.zetasql.FunctionSignatureProtoB\x02\x18\x01\x12\x33\n\nsignatures\x18\x0c \x03(\x0b\x32\x1f.zetasql.FunctionSignatureProto\x12\x39\n\x07options\x18\t \x01(\x0b\x32(.zetasql.TableValuedFunctionOptionsProto\x12<\n\x04type\x18\x03 \x01(\x0e\x32..zetasql.FunctionEnums.TableValuedFunctionType\x12\x35\n\nvolatility\x18\x08 \x01(\x0e\x32!.zetasql.FunctionEnums.Volatility\x12@\n\x15parse_resume_location\x18\x06 \x01(\x0b\x32!.zetasql.ParseResumeLocationProto\x12\x15\n\rargument_name\x18\x05 \x03(\t\x12\x16\n\x0e\x63ustom_context\x18\x07 \x01(\t\x12\x41\n\x12\x61nonymization_info\x18\n \x01(\x0b\x32%.zetasql.SimpleAnonymizationInfoProto\x12\x45\n\x11statement_context\x18\x0b \x01(\x0e\x32\x19.zetasql.StatementContext:\x0f\x43ONTEXT_DEFAULT\x12\r\n\x05group\x18\r \x01(\t\"\xd2\x02\n\x10TVFArgumentProto\x12\x34\n\x0fscalar_argument\x18\x01 \x01(\x0b\x32\x1b.zetasql.ValueWithTypeProto\x12\x34\n\x11relation_argument\x18\x02 \x01(\x0b\x32\x19.zetasql.TVFRelationProto\x12.\n\x0emodel_argument\x18\x03 \x01(\x0b\x32\x16.zetasql.TVFModelProto\x12\x38\n\x13\x63onnection_argument\x18\x04 \x01(\x0b\x32\x1b.zetasql.TVFConnectionProto\x12\x38\n\x13\x64\x65scriptor_argument\x18\x05 \x01(\x0b\x32\x1b.zetasql.TVFDescriptorProto\x12.\n\x0egraph_argument\x18\x06 \x01(\x0b\x32\x16.zetasql.TVFGraphProto\"k\n\x18TVFSignatureOptionsProto\x12O\n\x1e\x61\x64\x64itional_deprecation_warning\x18\x01 \x03(\x0b\x32\'.zetasql.FreestandingDeprecationWarning\"\xf6\x01\n\x11TVFSignatureProto\x12+\n\x08\x61rgument\x18\x01 \x03(\x0b\x32\x19.zetasql.TVFArgumentProto\x12\x32\n\x07options\x18\x03 \x01(\x0b\x32!.zetasql.TVFSignatureOptionsProto\x12\x32\n\routput_schema\x18\x02 \x01(\x0b\x32\x19.zetasql.TVFRelationProtoH\x00\x12\x35\n\x13output_table_schema\x18\x04 \x01(\x0b\x32\x16.zetasql.TableRefProtoH\x00\x42\x15\n\x13output_schema_oneof\"W\n\x0eProcedureProto\x12\x11\n\tname_path\x18\x01 \x03(\t\x12\x32\n\tsignature\x18\x02 \x01(\x0b\x32\x1f.zetasql.FunctionSignatureProtoB$\n\x12\x63om.google.zetasqlB\x0e\x46unctionProtos')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.proto.function_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022com.google.zetasqlB\016FunctionProtos'
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO'].fields_by_name['must_be_constant']._loaded_options = None
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO'].fields_by_name['must_be_constant']._serialized_options = b'\030\001'
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO'].fields_by_name['argument_name_is_mandatory']._loaded_options = None
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO'].fields_by_name['argument_name_is_mandatory']._serialized_options = b'\030\001'
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO'].fields_by_name['must_be_constant_expression']._loaded_options = None
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO'].fields_by_name['must_be_constant_expression']._serialized_options = b'\030\001'
  _globals['_TABLEVALUEDFUNCTIONPROTO'].fields_by_name['signature']._loaded_options = None
  _globals['_TABLEVALUEDFUNCTIONPROTO'].fields_by_name['signature']._serialized_options = b'\030\001'
  _globals['_TVFRELATIONCOLUMNPROTO']._serialized_start=482
  _globals['_TVFRELATIONCOLUMNPROTO']._serialized_end=771
  _globals['_TVFRELATIONPROTO']._serialized_start=773
  _globals['_TVFRELATIONPROTO']._serialized_end=871
  _globals['_TVFMODELPROTO']._serialized_start=873
  _globals['_TVFMODELPROTO']._serialized_end=921
  _globals['_TVFCONNECTIONPROTO']._serialized_start=923
  _globals['_TVFCONNECTIONPROTO']._serialized_end=976
  _globals['_TVFDESCRIPTORPROTO']._serialized_start=978
  _globals['_TVFDESCRIPTORPROTO']._serialized_end=1019
  _globals['_TVFGRAPHPROTO']._serialized_start=1021
  _globals['_TVFGRAPHPROTO']._serialized_end=1069
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO']._serialized_start=1072
  _globals['_FUNCTIONARGUMENTTYPEOPTIONSPROTO']._serialized_end=2478
  _globals['_ARGUMENTTYPELAMBDAPROTO']._serialized_start=2481
  _globals['_ARGUMENTTYPELAMBDAPROTO']._serialized_end=2610
  _globals['_FUNCTIONARGUMENTTYPEPROTO']._serialized_start=2613
  _globals['_FUNCTIONARGUMENTTYPEPROTO']._serialized_end=2855
  _globals['_FUNCTIONSIGNATUREREWRITEOPTIONSPROTO']._serialized_start=2858
  _globals['_FUNCTIONSIGNATUREREWRITEOPTIONSPROTO']._serialized_end=3038
  _globals['_FUNCTIONSIGNATUREOPTIONSPROTO']._serialized_start=3041
  _globals['_FUNCTIONSIGNATUREOPTIONSPROTO']._serialized_end=3464
  _globals['_FUNCTIONSIGNATUREPROTO']._serialized_start=3467
  _globals['_FUNCTIONSIGNATUREPROTO']._serialized_end=3679
  _globals['_FUNCTIONOPTIONSPROTO']._serialized_start=3682
  _globals['_FUNCTIONOPTIONSPROTO']._serialized_end=4504
  _globals['_FUNCTIONPROTO']._serialized_start=4507
  _globals['_FUNCTIONPROTO']._serialized_end=4981
  _globals['_RESOLVEDFUNCTIONCALLINFOPROTO']._serialized_start=4983
  _globals['_RESOLVEDFUNCTIONCALLINFOPROTO']._serialized_end=5014
  _globals['_TABLEVALUEDFUNCTIONOPTIONSPROTO']._serialized_start=5017
  _globals['_TABLEVALUEDFUNCTIONOPTIONSPROTO']._serialized_end=5151
  _globals['_TABLEVALUEDFUNCTIONPROTO']._serialized_start=5154
  _globals['_TABLEVALUEDFUNCTIONPROTO']._serialized_end=5750
  _globals['_TVFARGUMENTPROTO']._serialized_start=5753
  _globals['_TVFARGUMENTPROTO']._serialized_end=6091
  _globals['_TVFSIGNATUREOPTIONSPROTO']._serialized_start=6093
  _globals['_TVFSIGNATUREOPTIONSPROTO']._serialized_end=6200
  _globals['_TVFSIGNATUREPROTO']._serialized_start=6203
  _globals['_TVFSIGNATUREPROTO']._serialized_end=6449
  _globals['_PROCEDUREPROTO']._serialized_start=6451
  _globals['_PROCEDUREPROTO']._serialized_end=6538
# @@protoc_insertion_point(module_scope)



"""
ResolvedNode Visitor Pattern

Provides a visitor class for traversing ResolvedNode hierarchies with:
- Dynamic visit_{type} method dispatch using MRO (most specific first)
- descend() for controlled child node traversal
- Caching optimization for field metadata and method dispatch
"""

from zetasql.api.tree_visitor import TreeVisitor
from zetasql.types.proto_model import ResolvedNode


class ResolvedNodeVisitor(TreeVisitor[ResolvedNode]):
    """
    Base class for visiting ResolvedNode trees.

    Subclasses can override visit_{NodeType} methods to handle specific node types.
    The visitor will automatically dispatch to the most specific method available
    based on the node's MRO (Method Resolution Order).

    Example:
        >>> class MyVisitor(ResolvedNodeVisitor):
        ...     def visit_ResolvedExpr(self, node: ResolvedExpr) -> None:
        ...         print(f"Visiting expression: {type(node).__name__}")
        ...         self.descend(node)  # Visit children
        ...
        ...     def visit_ResolvedLiteral(self, node: ResolvedLiteral) -> None:
        ...         print(f"Literal value: {node.value}")
        ...         # Don't call descend() to stop here
        ...
        ...     def default_visit(self, node: ResolvedNode) -> None:
        ...         print(f"Default: {type(node).__name__}")
        ...         self.descend(node)

    Usage:
        >>> visitor = MyVisitor()
        >>> visitor.visit(resolved_root)
    """

    pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/public/formatter_options.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/public/formatter_options.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&zetasql/public/formatter_options.proto\x12\x07zetasql\"\xd5\x02\n\x15\x46ormatterOptionsProto\x12\x15\n\rnew_line_type\x18\x01 \x01(\t\x12\x19\n\x11line_length_limit\x18\x02 \x01(\x05\x12\x1a\n\x12indentation_spaces\x18\x03 \x01(\x05\x12\x1c\n\x14\x61llow_invalid_tokens\x18\x04 \x01(\x08\x12\x1b\n\x13\x63\x61pitalize_keywords\x18\x05 \x01(\x08\x12\x1c\n\x14preserve_line_breaks\x18\x06 \x01(\x08\x12\x1c\n\x14\x65xpand_format_ranges\x18\x07 \x01(\x08\x12\x1d\n\x15\x65nforce_single_quotes\x18\x08 \x01(\x08\x12\x1c\n\x14\x63\x61pitalize_functions\x18\t \x01(\x08\x12!\n\x19\x66ormat_structured_strings\x18\n \x01(\x08\x12\x17\n\x0f\x66ormat_comments\x18\x0b \x01(\x08\"1\n\x13\x46ormatterRangeProto\x12\r\n\x05start\x18\x01 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x05\x42\x14\n\x12\x63om.google.zetasql')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.public.formatter_options_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022com.google.zetasql'
  _globals['_FORMATTEROPTIONSPROTO']._serialized_start=52
  _globals['_FORMATTEROPTIONSPROTO']._serialized_end=393
  _globals['_FORMATTERRANGEPROTO']._serialized_start=395
  _globals['_FORMATTERRANGEPROTO']._serialized_end=444
# @@protoc_insertion_point(module_scope)



"""
ASTNode Visitor Pattern

Provides a visitor class for traversing ASTNode hierarchies with:
- Dynamic visit_{type} method dispatch using MRO (most specific first)
- descend() for controlled child node traversal
- Caching optimization for field metadata and method dispatch
"""

from zetasql.api.tree_visitor import TreeVisitor
from zetasql.types.proto_model import ASTNode


class ASTNodeVisitor(TreeVisitor[ASTNode]):
    """
    Base class for visiting ASTNode trees.

    Subclasses can override visit_{NodeType} methods to handle specific node types.
    The visitor will automatically dispatch to the most specific method available
    based on the node's MRO (Method Resolution Order).

    Example:
        >>> class MyVisitor(ASTNodeVisitor):
        ...     def visit_ASTExpression(self, node: ASTExpression) -> None:
        ...         print(f"Visiting expression: {type(node).__name__}")
        ...         self.descend(node)  # Visit children
        ...
        ...     def visit_ASTBinaryExpression(self, node: ASTBinaryExpression) -> None:
        ...         print(f"Binary: {node.op}")
        ...         # Don't call descend() to stop here
        ...
        ...     def default_visit(self, node: ASTNode) -> None:
        ...         print(f"Default: {type(node).__name__}")
        ...         self.descend(node)

    Usage:
        >>> visitor = MyVisitor()
        >>> visitor.visit(ast_root)
    """

    pass



from pathlib import Path


def get_wasm_path() -> Path:
    """Get the path to the zetasql.wasm file.
    
    Returns:
        Path: Absolute path to the zetasql.wasm binary file.
        
    Example:
        >>> from zetasql.wasi import get_wasm_path
        >>> wasm_path = get_wasm_path()
        >>> print(wasm_path)
        /path/to/site-packages/zetasql/wasi/zetasql.wasm
    """
    return Path(__file__).parent / "zetasql.wasm"


__all__ = ["get_wasm_path"]



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class SimpleValueProto(_message.Message):
    __slots__ = ("int64_value", "string_value", "bool_value", "double_value", "bytes_value", "__SimpleValueProto__switch_must_have_a_default")
    INT64_VALUE_FIELD_NUMBER: _ClassVar[int]
    STRING_VALUE_FIELD_NUMBER: _ClassVar[int]
    BOOL_VALUE_FIELD_NUMBER: _ClassVar[int]
    DOUBLE_VALUE_FIELD_NUMBER: _ClassVar[int]
    BYTES_VALUE_FIELD_NUMBER: _ClassVar[int]
    __SIMPLEVALUEPROTO__SWITCH_MUST_HAVE_A_DEFAULT_FIELD_NUMBER: _ClassVar[int]
    int64_value: int
    string_value: str
    bool_value: bool
    double_value: float
    bytes_value: bytes
    __SimpleValueProto__switch_must_have_a_default: bool
    def __init__(self, int64_value: _Optional[int] = ..., string_value: _Optional[str] = ..., bool_value: bool = ..., double_value: _Optional[float] = ..., bytes_value: _Optional[bytes] = ..., __SimpleValueProto__switch_must_have_a_default: bool = ...) -> None: ...



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PerModuleOptions(_message.Message):
    __slots__ = ("udf_server_address", "udf_namespace", "udf_server_import_mode", "udf_scaling_factor")
    class UdfServerImportMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNKNOWN: _ClassVar[PerModuleOptions.UdfServerImportMode]
        SERVER_ADDRESS_FROM_MODULE: _ClassVar[PerModuleOptions.UdfServerImportMode]
        CALLER_PROVIDED: _ClassVar[PerModuleOptions.UdfServerImportMode]
        MANUAL: _ClassVar[PerModuleOptions.UdfServerImportMode]
    UNKNOWN: PerModuleOptions.UdfServerImportMode
    SERVER_ADDRESS_FROM_MODULE: PerModuleOptions.UdfServerImportMode
    CALLER_PROVIDED: PerModuleOptions.UdfServerImportMode
    MANUAL: PerModuleOptions.UdfServerImportMode
    UDF_SERVER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    UDF_NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    UDF_SERVER_IMPORT_MODE_FIELD_NUMBER: _ClassVar[int]
    UDF_SCALING_FACTOR_FIELD_NUMBER: _ClassVar[int]
    udf_server_address: str
    udf_namespace: str
    udf_server_import_mode: PerModuleOptions.UdfServerImportMode
    udf_scaling_factor: float
    def __init__(self, udf_server_address: _Optional[str] = ..., udf_namespace: _Optional[str] = ..., udf_server_import_mode: _Optional[_Union[PerModuleOptions.UdfServerImportMode, str]] = ..., udf_scaling_factor: _Optional[float] = ...) -> None: ...

class ModuleOptions(_message.Message):
    __slots__ = ("global_options", "per_module_options")
    class PerModuleOptionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: PerModuleOptions
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[PerModuleOptions, _Mapping]] = ...) -> None: ...
    GLOBAL_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    PER_MODULE_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    global_options: PerModuleOptions
    per_module_options: _containers.MessageMap[str, PerModuleOptions]
    def __init__(self, global_options: _Optional[_Union[PerModuleOptions, _Mapping]] = ..., per_module_options: _Optional[_Mapping[str, PerModuleOptions]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/public/information_schema/property_graph.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/public/information_schema/property_graph.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6zetasql/public/information_schema/property_graph.proto\x12\x07zetasql\"\xa7\x02\n\x12PropertyGraphProto\x12\x0f\n\x07\x63\x61talog\x18\x01 \x01(\t\x12\x0e\n\x06schema\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x34\n\x0bnode_tables\x18\x04 \x03(\x0b\x32\x1f.zetasql.GraphElementTableProto\x12\x34\n\x0b\x65\x64ge_tables\x18\x05 \x03(\x0b\x32\x1f.zetasql.GraphElementTableProto\x12/\n\x06labels\x18\x06 \x03(\x0b\x32\x1f.zetasql.GraphElementLabelProto\x12\x45\n\x15property_declarations\x18\x07 \x03(\x0b\x32&.zetasql.GraphPropertyDeclarationProto\"\x8d\x04\n\x16GraphElementTableProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x32\n\x04kind\x18\x02 \x01(\x0e\x32$.zetasql.GraphElementTableProto.Kind\x12\x19\n\x11\x62\x61se_catalog_name\x18\x03 \x01(\t\x12\x18\n\x10\x62\x61se_schema_name\x18\x04 \x01(\t\x12\x17\n\x0f\x62\x61se_table_name\x18\x05 \x01(\t\x12\x13\n\x0bkey_columns\x18\x06 \x03(\t\x12\x13\n\x0blabel_names\x18\x07 \x03(\t\x12\x43\n\x14property_definitions\x18\x08 \x03(\x0b\x32%.zetasql.GraphPropertyDefinitionProto\x12\x1a\n\x12\x64ynamic_label_expr\x18\x0b \x01(\t\x12\x1d\n\x15\x64ynamic_property_expr\x18\x0c \x01(\t\x12@\n\x11source_node_table\x18\t \x01(\x0b\x32%.zetasql.GraphNodeTableReferenceProto\x12\x45\n\x16\x64\x65stination_node_table\x18\n \x01(\x0b\x32%.zetasql.GraphNodeTableReferenceProto\"0\n\x04Kind\x12\x14\n\x10KIND_UNSPECIFIED\x10\x00\x12\x08\n\x04NODE\x10\x01\x12\x08\n\x04\x45\x44GE\x10\x02\"o\n\x1cGraphNodeTableReferenceProto\x12\x17\n\x0fnode_table_name\x18\x01 \x01(\t\x12\x1a\n\x12\x65\x64ge_table_columns\x18\x02 \x03(\t\x12\x1a\n\x12node_table_columns\x18\x03 \x03(\t\"J\n\x16GraphElementLabelProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\"\n\x1aproperty_declaration_names\x18\x02 \x03(\t\";\n\x1dGraphPropertyDeclarationProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\"_\n\x1cGraphPropertyDefinitionProto\x12!\n\x19property_declaration_name\x18\x01 \x01(\t\x12\x1c\n\x14value_expression_sql\x18\x02 \x01(\tB)\n\x12\x63om.google.zetasqlB\x13PropertyGraphProtos')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.public.information_schema.property_graph_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022com.google.zetasqlB\023PropertyGraphProtos'
  _globals['_PROPERTYGRAPHPROTO']._serialized_start=68
  _globals['_PROPERTYGRAPHPROTO']._serialized_end=363
  _globals['_GRAPHELEMENTTABLEPROTO']._serialized_start=366
  _globals['_GRAPHELEMENTTABLEPROTO']._serialized_end=891
  _globals['_GRAPHELEMENTTABLEPROTO_KIND']._serialized_start=843
  _globals['_GRAPHELEMENTTABLEPROTO_KIND']._serialized_end=891
  _globals['_GRAPHNODETABLEREFERENCEPROTO']._serialized_start=893
  _globals['_GRAPHNODETABLEREFERENCEPROTO']._serialized_end=1004
  _globals['_GRAPHELEMENTLABELPROTO']._serialized_start=1006
  _globals['_GRAPHELEMENTLABELPROTO']._serialized_end=1080
  _globals['_GRAPHPROPERTYDECLARATIONPROTO']._serialized_start=1082
  _globals['_GRAPHPROPERTYDECLARATIONPROTO']._serialized_end=1141
  _globals['_GRAPHPROPERTYDEFINITIONPROTO']._serialized_start=1143
  _globals['_GRAPHPROPERTYDEFINITIONPROTO']._serialized_end=1238
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class ConstnessLevelProto(_message.Message):
    __slots__ = ()
    class Level(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        CONSTNESS_UNSPECIFIED: _ClassVar[ConstnessLevelProto.Level]
        FOREVER_CONST: _ClassVar[ConstnessLevelProto.Level]
        ANALYSIS_CONST: _ClassVar[ConstnessLevelProto.Level]
        LEGACY_LITERAL_OR_PARAMETER: _ClassVar[ConstnessLevelProto.Level]
        LEGACY_CONSTANT_EXPRESSION: _ClassVar[ConstnessLevelProto.Level]
    CONSTNESS_UNSPECIFIED: ConstnessLevelProto.Level
    FOREVER_CONST: ConstnessLevelProto.Level
    ANALYSIS_CONST: ConstnessLevelProto.Level
    LEGACY_LITERAL_OR_PARAMETER: ConstnessLevelProto.Level
    LEGACY_CONSTANT_EXPRESSION: ConstnessLevelProto.Level
    def __init__(self) -> None: ...



from google.protobuf import descriptor_pb2 as _descriptor_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf.internal import python_message as _python_message
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class TypeKind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    __TypeKind__switch_must_have_a_default__: _ClassVar[TypeKind]
    TYPE_UNKNOWN: _ClassVar[TypeKind]
    TYPE_INT32: _ClassVar[TypeKind]
    TYPE_INT64: _ClassVar[TypeKind]
    TYPE_UINT32: _ClassVar[TypeKind]
    TYPE_UINT64: _ClassVar[TypeKind]
    TYPE_BOOL: _ClassVar[TypeKind]
    TYPE_FLOAT: _ClassVar[TypeKind]
    TYPE_DOUBLE: _ClassVar[TypeKind]
    TYPE_STRING: _ClassVar[TypeKind]
    TYPE_BYTES: _ClassVar[TypeKind]
    TYPE_DATE: _ClassVar[TypeKind]
    TYPE_TIMESTAMP: _ClassVar[TypeKind]
    TYPE_ENUM: _ClassVar[TypeKind]
    TYPE_ARRAY: _ClassVar[TypeKind]
    TYPE_STRUCT: _ClassVar[TypeKind]
    TYPE_PROTO: _ClassVar[TypeKind]
    TYPE_TIME: _ClassVar[TypeKind]
    TYPE_DATETIME: _ClassVar[TypeKind]
    TYPE_GEOGRAPHY: _ClassVar[TypeKind]
    TYPE_NUMERIC: _ClassVar[TypeKind]
    TYPE_BIGNUMERIC: _ClassVar[TypeKind]
    TYPE_EXTENDED: _ClassVar[TypeKind]
    TYPE_JSON: _ClassVar[TypeKind]
    TYPE_INTERVAL: _ClassVar[TypeKind]
    TYPE_TOKENLIST: _ClassVar[TypeKind]
    TYPE_RANGE: _ClassVar[TypeKind]
    TYPE_GRAPH_ELEMENT: _ClassVar[TypeKind]
    TYPE_GRAPH_PATH: _ClassVar[TypeKind]
    TYPE_MAP: _ClassVar[TypeKind]
    TYPE_UUID: _ClassVar[TypeKind]
    TYPE_MEASURE: _ClassVar[TypeKind]
    TYPE_ROW: _ClassVar[TypeKind]
__TypeKind__switch_must_have_a_default__: TypeKind
TYPE_UNKNOWN: TypeKind
TYPE_INT32: TypeKind
TYPE_INT64: TypeKind
TYPE_UINT32: TypeKind
TYPE_UINT64: TypeKind
TYPE_BOOL: TypeKind
TYPE_FLOAT: TypeKind
TYPE_DOUBLE: TypeKind
TYPE_STRING: TypeKind
TYPE_BYTES: TypeKind
TYPE_DATE: TypeKind
TYPE_TIMESTAMP: TypeKind
TYPE_ENUM: TypeKind
TYPE_ARRAY: TypeKind
TYPE_STRUCT: TypeKind
TYPE_PROTO: TypeKind
TYPE_TIME: TypeKind
TYPE_DATETIME: TypeKind
TYPE_GEOGRAPHY: TypeKind
TYPE_NUMERIC: TypeKind
TYPE_BIGNUMERIC: TypeKind
TYPE_EXTENDED: TypeKind
TYPE_JSON: TypeKind
TYPE_INTERVAL: TypeKind
TYPE_TOKENLIST: TypeKind
TYPE_RANGE: TypeKind
TYPE_GRAPH_ELEMENT: TypeKind
TYPE_GRAPH_PATH: TypeKind
TYPE_MAP: TypeKind
TYPE_UUID: TypeKind
TYPE_MEASURE: TypeKind
TYPE_ROW: TypeKind
OPAQUE_ENUM_TYPE_OPTIONS_FIELD_NUMBER: _ClassVar[int]
opaque_enum_type_options: _descriptor.FieldDescriptor
OPAQUE_ENUM_VALUE_OPTIONS_FIELD_NUMBER: _ClassVar[int]
opaque_enum_value_options: _descriptor.FieldDescriptor

class TypeProto(_message.Message):
    __slots__ = ("type_kind", "array_type", "struct_type", "proto_type", "enum_type", "range_type", "graph_element_type", "graph_path_type", "map_type", "measure_type", "file_descriptor_set", "extended_type_name")
    Extensions: _python_message._ExtensionDict
    TYPE_KIND_FIELD_NUMBER: _ClassVar[int]
    ARRAY_TYPE_FIELD_NUMBER: _ClassVar[int]
    STRUCT_TYPE_FIELD_NUMBER: _ClassVar[int]
    PROTO_TYPE_FIELD_NUMBER: _ClassVar[int]
    ENUM_TYPE_FIELD_NUMBER: _ClassVar[int]
    RANGE_TYPE_FIELD_NUMBER: _ClassVar[int]
    GRAPH_ELEMENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    GRAPH_PATH_TYPE_FIELD_NUMBER: _ClassVar[int]
    MAP_TYPE_FIELD_NUMBER: _ClassVar[int]
    MEASURE_TYPE_FIELD_NUMBER: _ClassVar[int]
    FILE_DESCRIPTOR_SET_FIELD_NUMBER: _ClassVar[int]
    EXTENDED_TYPE_NAME_FIELD_NUMBER: _ClassVar[int]
    type_kind: TypeKind
    array_type: ArrayTypeProto
    struct_type: StructTypeProto
    proto_type: ProtoTypeProto
    enum_type: EnumTypeProto
    range_type: RangeTypeProto
    graph_element_type: GraphElementTypeProto
    graph_path_type: GraphPathTypeProto
    map_type: MapTypeProto
    measure_type: MeasureTypeProto
    file_descriptor_set: _containers.RepeatedCompositeFieldContainer[_descriptor_pb2.FileDescriptorSet]
    extended_type_name: str
    def __init__(self, type_kind: _Optional[_Union[TypeKind, str]] = ..., array_type: _Optional[_Union[ArrayTypeProto, _Mapping]] = ..., struct_type: _Optional[_Union[StructTypeProto, _Mapping]] = ..., proto_type: _Optional[_Union[ProtoTypeProto, _Mapping]] = ..., enum_type: _Optional[_Union[EnumTypeProto, _Mapping]] = ..., range_type: _Optional[_Union[RangeTypeProto, _Mapping]] = ..., graph_element_type: _Optional[_Union[GraphElementTypeProto, _Mapping]] = ..., graph_path_type: _Optional[_Union[GraphPathTypeProto, _Mapping]] = ..., map_type: _Optional[_Union[MapTypeProto, _Mapping]] = ..., measure_type: _Optional[_Union[MeasureTypeProto, _Mapping]] = ..., file_descriptor_set: _Optional[_Iterable[_Union[_descriptor_pb2.FileDescriptorSet, _Mapping]]] = ..., extended_type_name: _Optional[str] = ...) -> None: ...

class ArrayTypeProto(_message.Message):
    __slots__ = ("element_type",)
    ELEMENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    element_type: TypeProto
    def __init__(self, element_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...

class StructFieldProto(_message.Message):
    __slots__ = ("field_name", "field_type")
    FIELD_NAME_FIELD_NUMBER: _ClassVar[int]
    FIELD_TYPE_FIELD_NUMBER: _ClassVar[int]
    field_name: str
    field_type: TypeProto
    def __init__(self, field_name: _Optional[str] = ..., field_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...

class StructTypeProto(_message.Message):
    __slots__ = ("field",)
    FIELD_FIELD_NUMBER: _ClassVar[int]
    field: _containers.RepeatedCompositeFieldContainer[StructFieldProto]
    def __init__(self, field: _Optional[_Iterable[_Union[StructFieldProto, _Mapping]]] = ...) -> None: ...

class MapTypeProto(_message.Message):
    __slots__ = ("key_type", "value_type")
    KEY_TYPE_FIELD_NUMBER: _ClassVar[int]
    VALUE_TYPE_FIELD_NUMBER: _ClassVar[int]
    key_type: TypeProto
    value_type: TypeProto
    def __init__(self, key_type: _Optional[_Union[TypeProto, _Mapping]] = ..., value_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...

class ProtoTypeProto(_message.Message):
    __slots__ = ("proto_name", "proto_file_name", "file_descriptor_set_index", "catalog_name_path")
    PROTO_NAME_FIELD_NUMBER: _ClassVar[int]
    PROTO_FILE_NAME_FIELD_NUMBER: _ClassVar[int]
    FILE_DESCRIPTOR_SET_INDEX_FIELD_NUMBER: _ClassVar[int]
    CATALOG_NAME_PATH_FIELD_NUMBER: _ClassVar[int]
    proto_name: str
    proto_file_name: str
    file_descriptor_set_index: int
    catalog_name_path: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, proto_name: _Optional[str] = ..., proto_file_name: _Optional[str] = ..., file_descriptor_set_index: _Optional[int] = ..., catalog_name_path: _Optional[_Iterable[str]] = ...) -> None: ...

class EnumTypeProto(_message.Message):
    __slots__ = ("enum_name", "enum_file_name", "file_descriptor_set_index", "catalog_name_path", "is_opaque")
    ENUM_NAME_FIELD_NUMBER: _ClassVar[int]
    ENUM_FILE_NAME_FIELD_NUMBER: _ClassVar[int]
    FILE_DESCRIPTOR_SET_INDEX_FIELD_NUMBER: _ClassVar[int]
    CATALOG_NAME_PATH_FIELD_NUMBER: _ClassVar[int]
    IS_OPAQUE_FIELD_NUMBER: _ClassVar[int]
    enum_name: str
    enum_file_name: str
    file_descriptor_set_index: int
    catalog_name_path: _containers.RepeatedScalarFieldContainer[str]
    is_opaque: bool
    def __init__(self, enum_name: _Optional[str] = ..., enum_file_name: _Optional[str] = ..., file_descriptor_set_index: _Optional[int] = ..., catalog_name_path: _Optional[_Iterable[str]] = ..., is_opaque: bool = ...) -> None: ...

class RangeTypeProto(_message.Message):
    __slots__ = ("element_type",)
    ELEMENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    element_type: TypeProto
    def __init__(self, element_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...

class OpaqueEnumTypeOptions(_message.Message):
    __slots__ = ("sql_opaque_enum_name",)
    SQL_OPAQUE_ENUM_NAME_FIELD_NUMBER: _ClassVar[int]
    sql_opaque_enum_name: str
    def __init__(self, sql_opaque_enum_name: _Optional[str] = ...) -> None: ...

class OpaqueEnumValueOptions(_message.Message):
    __slots__ = ("invalid_enum_value",)
    INVALID_ENUM_VALUE_FIELD_NUMBER: _ClassVar[int]
    invalid_enum_value: bool
    def __init__(self, invalid_enum_value: bool = ...) -> None: ...

class GraphElementTypeProto(_message.Message):
    __slots__ = ("kind", "property_type", "graph_reference", "is_dynamic")
    class ElementKind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        KIND_INVALID: _ClassVar[GraphElementTypeProto.ElementKind]
        KIND_NODE: _ClassVar[GraphElementTypeProto.ElementKind]
        KIND_EDGE: _ClassVar[GraphElementTypeProto.ElementKind]
    KIND_INVALID: GraphElementTypeProto.ElementKind
    KIND_NODE: GraphElementTypeProto.ElementKind
    KIND_EDGE: GraphElementTypeProto.ElementKind
    class PropertyTypeProto(_message.Message):
        __slots__ = ("name", "value_type")
        NAME_FIELD_NUMBER: _ClassVar[int]
        VALUE_TYPE_FIELD_NUMBER: _ClassVar[int]
        name: str
        value_type: TypeProto
        def __init__(self, name: _Optional[str] = ..., value_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...
    KIND_FIELD_NUMBER: _ClassVar[int]
    PROPERTY_TYPE_FIELD_NUMBER: _ClassVar[int]
    GRAPH_REFERENCE_FIELD_NUMBER: _ClassVar[int]
    IS_DYNAMIC_FIELD_NUMBER: _ClassVar[int]
    kind: GraphElementTypeProto.ElementKind
    property_type: _containers.RepeatedCompositeFieldContainer[GraphElementTypeProto.PropertyTypeProto]
    graph_reference: _containers.RepeatedScalarFieldContainer[str]
    is_dynamic: bool
    def __init__(self, kind: _Optional[_Union[GraphElementTypeProto.ElementKind, str]] = ..., property_type: _Optional[_Iterable[_Union[GraphElementTypeProto.PropertyTypeProto, _Mapping]]] = ..., graph_reference: _Optional[_Iterable[str]] = ..., is_dynamic: bool = ...) -> None: ...

class GraphPathTypeProto(_message.Message):
    __slots__ = ("node_type", "edge_type")
    NODE_TYPE_FIELD_NUMBER: _ClassVar[int]
    EDGE_TYPE_FIELD_NUMBER: _ClassVar[int]
    node_type: GraphElementTypeProto
    edge_type: GraphElementTypeProto
    def __init__(self, node_type: _Optional[_Union[GraphElementTypeProto, _Mapping]] = ..., edge_type: _Optional[_Union[GraphElementTypeProto, _Mapping]] = ...) -> None: ...

class MeasureTypeProto(_message.Message):
    __slots__ = ("result_type",)
    RESULT_TYPE_FIELD_NUMBER: _ClassVar[int]
    result_type: TypeProto
    def __init__(self, result_type: _Optional[_Union[TypeProto, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/public/value.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/public/value.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1azetasql/public/value.proto\x12\x07zetasql\x1a\x1fgoogle/protobuf/timestamp.proto\"\xf7\n\n\nValueProto\x12\x15\n\x0bint32_value\x18\x01 \x01(\x05H\x00\x12\x15\n\x0bint64_value\x18\x02 \x01(\x03H\x00\x12\x16\n\x0cuint32_value\x18\x03 \x01(\rH\x00\x12\x16\n\x0cuint64_value\x18\x04 \x01(\x04H\x00\x12\x14\n\nbool_value\x18\x05 \x01(\x08H\x00\x12\x15\n\x0b\x66loat_value\x18\x06 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x07 \x01(\x01H\x00\x12\x16\n\x0cstring_value\x18\x08 \x01(\tH\x00\x12\x15\n\x0b\x62ytes_value\x18\t \x01(\x0cH\x00\x12\x14\n\ndate_value\x18\n \x01(\x05H\x00\x12\x14\n\nenum_value\x18\x0c \x01(\x05H\x00\x12\x30\n\x0b\x61rray_value\x18\r \x01(\x0b\x32\x19.zetasql.ValueProto.ArrayH\x00\x12\x32\n\x0cstruct_value\x18\x0e \x01(\x0b\x32\x1a.zetasql.ValueProto.StructH\x00\x12\x19\n\x0bproto_value\x18\x0f \x01(\x0c\x42\x02\x08\x01H\x00\x12\x35\n\x0ftimestamp_value\x18\x10 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x12\"\n\x14timestamp_pico_value\x18\x1d \x01(\x0c\x42\x02\x18\x01H\x00\x12\x43\n\x15timestamp_picos_value\x18\x1e \x01(\x0b\x32\".zetasql.ValueProto.TimestampPicosH\x00\x12\x36\n\x0e\x64\x61tetime_value\x18\x11 \x01(\x0b\x32\x1c.zetasql.ValueProto.DatetimeH\x00\x12\x14\n\ntime_value\x18\x12 \x01(\x03H\x00\x12\x19\n\x0fgeography_value\x18\x13 \x01(\x0cH\x00\x12\x17\n\rnumeric_value\x18\x14 \x01(\x0cH\x00\x12\x1a\n\x10\x62ignumeric_value\x18\x15 \x01(\x0cH\x00\x12\x14\n\njson_value\x18\x17 \x01(\tH\x00\x12\x18\n\x0einterval_value\x18\x18 \x01(\x0cH\x00\x12\x19\n\x0ftokenlist_value\x18\x19 \x01(\x0cH\x00\x12\x30\n\x0brange_value\x18\x1a \x01(\x0b\x32\x19.zetasql.ValueProto.RangeH\x00\x12\x14\n\nuuid_value\x18\x1b \x01(\x0cH\x00\x12,\n\tmap_value\x18\x1c \x01(\x0b\x32\x17.zetasql.ValueProto.MapH\x00\x12\x33\n(__ValueProto__switch_must_have_a_default\x18\xff\x01 \x01(\x08H\x00\x1a-\n\x05\x41rray\x12$\n\x07\x65lement\x18\x01 \x03(\x0b\x32\x13.zetasql.ValueProto\x1a,\n\x06Struct\x12\"\n\x05\x66ield\x18\x01 \x03(\x0b\x32\x13.zetasql.ValueProto\x1a=\n\x08\x44\x61tetime\x12\"\n\x1a\x62it_field_datetime_seconds\x18\x01 \x01(\x03\x12\r\n\x05nanos\x18\x02 \x01(\x05\x1aM\n\x05Range\x12\"\n\x05start\x18\x01 \x01(\x0b\x32\x13.zetasql.ValueProto\x12 \n\x03\x65nd\x18\x02 \x01(\x0b\x32\x13.zetasql.ValueProto\x1aP\n\x08MapEntry\x12 \n\x03key\x18\x01 \x01(\x0b\x32\x13.zetasql.ValueProto\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.zetasql.ValueProto\x1a\x32\n\x03Map\x12+\n\x05\x65ntry\x18\x01 \x03(\x0b\x32\x1c.zetasql.ValueProto.MapEntry\x1a\x30\n\x0eTimestampPicos\x12\x0f\n\x07seconds\x18\x01 \x01(\x03\x12\r\n\x05picos\x18\x02 \x01(\x03\x42\x07\n\x05valueJ\x04\x08\x0b\x10\x0cJ\x04\x08\x16\x10\x17\x42%\n\x12\x63om.google.zetasqlB\x0cZetaSQLValue\xf8\x01\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.public.value_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022com.google.zetasqlB\014ZetaSQLValue\370\001\001'
  _globals['_VALUEPROTO'].fields_by_name['proto_value']._loaded_options = None
  _globals['_VALUEPROTO'].fields_by_name['proto_value']._serialized_options = b'\010\001'
  _globals['_VALUEPROTO'].fields_by_name['timestamp_pico_value']._loaded_options = None
  _globals['_VALUEPROTO'].fields_by_name['timestamp_pico_value']._serialized_options = b'\030\001'
  _globals['_VALUEPROTO']._serialized_start=73
  _globals['_VALUEPROTO']._serialized_end=1472
  _globals['_VALUEPROTO_ARRAY']._serialized_start=1034
  _globals['_VALUEPROTO_ARRAY']._serialized_end=1079
  _globals['_VALUEPROTO_STRUCT']._serialized_start=1081
  _globals['_VALUEPROTO_STRUCT']._serialized_end=1125
  _globals['_VALUEPROTO_DATETIME']._serialized_start=1127
  _globals['_VALUEPROTO_DATETIME']._serialized_end=1188
  _globals['_VALUEPROTO_RANGE']._serialized_start=1190
  _globals['_VALUEPROTO_RANGE']._serialized_end=1267
  _globals['_VALUEPROTO_MAPENTRY']._serialized_start=1269
  _globals['_VALUEPROTO_MAPENTRY']._serialized_end=1349
  _globals['_VALUEPROTO_MAP']._serialized_start=1351
  _globals['_VALUEPROTO_MAP']._serialized_end=1401
  _globals['_VALUEPROTO_TIMESTAMPPICOS']._serialized_start=1403
  _globals['_VALUEPROTO_TIMESTAMPPICOS']._serialized_end=1451
# @@protoc_insertion_point(module_scope)



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class ParseLocationRangeProto(_message.Message):
    __slots__ = ("filename", "start", "end")
    FILENAME_FIELD_NUMBER: _ClassVar[int]
    START_FIELD_NUMBER: _ClassVar[int]
    END_FIELD_NUMBER: _ClassVar[int]
    filename: str
    start: int
    end: int
    def __init__(self, filename: _Optional[str] = ..., start: _Optional[int] = ..., end: _Optional[int] = ...) -> None: ...



import datetime

from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ExecutionStats(_message.Message):
    __slots__ = ("wall_time", "cpu_time", "stack_available_bytes", "stack_peak_used_bytes")
    class ParserVariant(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        PARSER_UNSPECIFIED: _ClassVar[ExecutionStats.ParserVariant]
        PARSER_BISON: _ClassVar[ExecutionStats.ParserVariant]
        PARSER_TEXTMAPPER: _ClassVar[ExecutionStats.ParserVariant]
    PARSER_UNSPECIFIED: ExecutionStats.ParserVariant
    PARSER_BISON: ExecutionStats.ParserVariant
    PARSER_TEXTMAPPER: ExecutionStats.ParserVariant
    WALL_TIME_FIELD_NUMBER: _ClassVar[int]
    CPU_TIME_FIELD_NUMBER: _ClassVar[int]
    STACK_AVAILABLE_BYTES_FIELD_NUMBER: _ClassVar[int]
    STACK_PEAK_USED_BYTES_FIELD_NUMBER: _ClassVar[int]
    wall_time: _duration_pb2.Duration
    cpu_time: _duration_pb2.Duration
    stack_available_bytes: int
    stack_peak_used_bytes: int
    def __init__(self, wall_time: _Optional[_Union[datetime.timedelta, _duration_pb2.Duration, _Mapping]] = ..., cpu_time: _Optional[_Union[datetime.timedelta, _duration_pb2.Duration, _Mapping]] = ..., stack_available_bytes: _Optional[int] = ..., stack_peak_used_bytes: _Optional[int] = ...) -> None: ...

class AnalyzerLogEntry(_message.Message):
    __slots__ = ("num_lexical_tokens", "overall_execution_stats", "execution_stats_by_op")
    class LoggedOperationCategory(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNKNOWN_LOGGED_OPERATION_CATEGORY: _ClassVar[AnalyzerLogEntry.LoggedOperationCategory]
        PARSER: _ClassVar[AnalyzerLogEntry.LoggedOperationCategory]
        RESOLVER: _ClassVar[AnalyzerLogEntry.LoggedOperationCategory]
        REWRITER: _ClassVar[AnalyzerLogEntry.LoggedOperationCategory]
        CATALOG_RESOLVER: _ClassVar[AnalyzerLogEntry.LoggedOperationCategory]
        VALIDATOR: _ClassVar[AnalyzerLogEntry.LoggedOperationCategory]
    UNKNOWN_LOGGED_OPERATION_CATEGORY: AnalyzerLogEntry.LoggedOperationCategory
    PARSER: AnalyzerLogEntry.LoggedOperationCategory
    RESOLVER: AnalyzerLogEntry.LoggedOperationCategory
    REWRITER: AnalyzerLogEntry.LoggedOperationCategory
    CATALOG_RESOLVER: AnalyzerLogEntry.LoggedOperationCategory
    VALIDATOR: AnalyzerLogEntry.LoggedOperationCategory
    class ExecutionStatsByOpEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: AnalyzerLogEntry.LoggedOperationCategory
        value: ExecutionStats
        def __init__(self, key: _Optional[_Union[AnalyzerLogEntry.LoggedOperationCategory, str]] = ..., value: _Optional[_Union[ExecutionStats, _Mapping]] = ...) -> None: ...
    NUM_LEXICAL_TOKENS_FIELD_NUMBER: _ClassVar[int]
    OVERALL_EXECUTION_STATS_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_STATS_BY_OP_FIELD_NUMBER: _ClassVar[int]
    num_lexical_tokens: int
    overall_execution_stats: ExecutionStats
    execution_stats_by_op: _containers.RepeatedCompositeFieldContainer[AnalyzerLogEntry.ExecutionStatsByOpEntry]
    def __init__(self, num_lexical_tokens: _Optional[int] = ..., overall_execution_stats: _Optional[_Union[ExecutionStats, _Mapping]] = ..., execution_stats_by_op: _Optional[_Iterable[_Union[AnalyzerLogEntry.ExecutionStatsByOpEntry, _Mapping]]] = ...) -> None: ...



"""
TreeVisitor - Generic Visitor Pattern for ProtoModel Trees

Provides a generic base visitor class for traversing ProtoModel hierarchies with:
- Dynamic visit_{type} method dispatch using MRO (most specific first)
- descend() for controlled child node traversal
- Class-level caching optimization for field metadata and method dispatch
"""

from dataclasses import fields
from typing import Any, ClassVar, Generic, TypeVar

from zetasql.types.proto_model import ProtoModel

# Type variable bound to ProtoModel for generic visitor
T = TypeVar("T", bound=ProtoModel)


class TreeVisitor(Generic[T]):
    """
    Generic base class for visiting ProtoModel trees.

    Subclasses can override visit_{NodeType} methods to handle specific node types.
    The visitor will automatically dispatch to the most specific method available
    based on the node's MRO (Method Resolution Order).

    Type Parameter:
        T: The ProtoModel subclass this visitor operates on (e.g., ASTNode, ResolvedNode)

    Example:
        >>> class MyVisitor(TreeVisitor[ASTNode]):
        ...     def visit_ASTExpression(self, node: ASTExpression) -> None:
        ...         print(f"Visiting expression: {type(node).__name__}")
        ...         self.descend(node)  # Visit children
        ...
        ...     def visit_ASTBinaryExpression(self, node: ASTBinaryExpression) -> None:
        ...         print(f"Binary: {node.op}")
        ...         # Don't call descend() to stop here
        ...
        ...     def default_visit(self, node: ASTNode) -> None:
        ...         print(f"Default: {type(node).__name__}")
        ...         self.descend(node)

    Usage:
        >>> visitor = MyVisitor()
        >>> visitor.visit(tree_root)
    """

    # Class-level caches shared across instances of the same visitor class
    # Each subclass gets its own cache dictionaries
    _field_cache: ClassVar[dict[type, list[tuple]]] = {}
    _method_cache: ClassVar[dict[type, str]] = {}  # Stores method names, not callables

    def __init_subclass__(cls, **kwargs):
        """Initialize separate caches for each subclass."""
        super().__init_subclass__(**kwargs)
        # Each subclass gets its own cache dictionaries
        cls._field_cache = {}
        cls._method_cache = {}

    def visit(self, node: T) -> Any:
        """
        Visit a node by dispatching to the most specific visit_{type} method.

        Searches through the node's MRO to find the most specific visitor method.
        Falls back to default_visit if no specific method is found.

        Args:
            node: The ProtoModel node to visit

        Returns:
            The return value of the visitor method
        """
        if node is None:
            return None

        node_type = type(node)

        # Check method cache first
        if node_type in self.__class__._method_cache:
            method_name = self.__class__._method_cache[node_type]
            method = getattr(self, method_name)
            return method(node)

        # Find the most specific visitor method using MRO
        method_name = self._find_visitor_method_name(node_type)

        # Cache the method name for future visits
        self.__class__._method_cache[node_type] = method_name

        method = getattr(self, method_name)
        return method(node)

    def _find_visitor_method_name(self, node_type: type) -> str:
        """
        Find the most specific visitor method name for a node type.

        Walks the MRO from most specific to least specific, looking for
        visit_{ClassName} methods. Stops at ProtoModel base class.

        Args:
            node_type: The type of node to find a visitor for

        Returns:
            The visitor method name (or 'default_visit' if none found)
        """
        # Walk MRO from most specific to most general
        for cls in node_type.__mro__:
            # Stop at ProtoModel - don't go beyond our type hierarchy
            if cls is ProtoModel:
                break

            method_name = f"visit_{cls.__name__}"
            if hasattr(self, method_name):
                return method_name

        # No specific visitor found, use default
        return "default_visit"

    def default_visit(self, node: T) -> Any:
        """
        Default visitor method called when no specific visit_{type} exists.

        By default, automatically descends into child nodes. Override this to
        provide custom default behavior. If you override this, remember to call
        self.descend(node) if you want to continue traversing children.

        Args:
            node: The ProtoModel node being visited

        Returns:
            None by default
        """
        self.descend(node)

    def descend(self, node: T) -> None:
        """
        Visit all child nodes of the given node.

        Only visits fields that are ProtoModel instances (is_message=True).
        Handles both single nodes and lists of nodes.

        Call this method in your visit_{type} methods to traverse child nodes.
        If you don't call this, the traversal stops at the current node.

        Args:
            node: The ProtoModel node whose children should be visited
        """
        if node is None:
            return

        node_type = type(node)

        # Check cache first
        if node_type not in self.__class__._field_cache:
            self.__class__._field_cache[node_type] = self._get_message_fields(node_type)

        message_fields = self.__class__._field_cache[node_type]

        # Visit each child node
        for field_name, is_repeated in message_fields:
            value = getattr(node, field_name, None)

            if value is None:
                continue

            if is_repeated:
                # Handle list of child nodes
                for item in value:
                    if isinstance(item, ProtoModel):
                        self.visit(item)
            else:
                # Handle single child node
                if isinstance(value, ProtoModel):
                    self.visit(value)

    def _get_message_fields(self, node_type: type) -> list[tuple]:
        """
        Extract fields that are ProtoModel children (is_message=True).

        Args:
            node_type: The type of node to analyze

        Returns:
            List of (field_name, is_repeated) tuples for ProtoModel fields
        """
        message_fields = []

        # Build a combined field map from the entire inheritance chain
        combined_field_map = {}
        for cls in reversed(node_type.__mro__):
            # Skip ProtoModel itself and classes before it (object, etc.)
            if not issubclass(cls, ProtoModel) or cls is ProtoModel:
                continue

            # Merge this class's field map (child overrides parent)
            if hasattr(cls, "_PROTO_FIELD_MAP"):
                combined_field_map.update(cls._PROTO_FIELD_MAP)

        # Get all dataclass fields
        for field in fields(node_type):
            field_name = field.name

            # Skip private fields
            if field_name.startswith("_"):
                continue

            # Check if this field is a message (ProtoModel child)
            if field_name in combined_field_map:
                field_meta = combined_field_map[field_name]
                if field_meta.get("is_message", False):
                    is_repeated = field_meta.get("is_repeated", False)
                    message_fields.append((field_name, is_repeated))

        return message_fields



from zetasql.wasi._pb2.zetasql.public import type_pb2 as _type_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class ArrayZipEnums(_message.Message):
    __slots__ = ()
    class ArrayZipMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        ARRAY_ZIP_MODE_INVALID: _ClassVar[ArrayZipEnums.ArrayZipMode]
        PAD: _ClassVar[ArrayZipEnums.ArrayZipMode]
        TRUNCATE: _ClassVar[ArrayZipEnums.ArrayZipMode]
        STRICT: _ClassVar[ArrayZipEnums.ArrayZipMode]
    ARRAY_ZIP_MODE_INVALID: ArrayZipEnums.ArrayZipMode
    PAD: ArrayZipEnums.ArrayZipMode
    TRUNCATE: ArrayZipEnums.ArrayZipMode
    STRICT: ArrayZipEnums.ArrayZipMode
    def __init__(self) -> None: ...



import datetime

from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ValueProto(_message.Message):
    __slots__ = ("int32_value", "int64_value", "uint32_value", "uint64_value", "bool_value", "float_value", "double_value", "string_value", "bytes_value", "date_value", "enum_value", "array_value", "struct_value", "proto_value", "timestamp_value", "timestamp_pico_value", "timestamp_picos_value", "datetime_value", "time_value", "geography_value", "numeric_value", "bignumeric_value", "json_value", "interval_value", "tokenlist_value", "range_value", "uuid_value", "map_value", "__ValueProto__switch_must_have_a_default")
    class Array(_message.Message):
        __slots__ = ("element",)
        ELEMENT_FIELD_NUMBER: _ClassVar[int]
        element: _containers.RepeatedCompositeFieldContainer[ValueProto]
        def __init__(self, element: _Optional[_Iterable[_Union[ValueProto, _Mapping]]] = ...) -> None: ...
    class Struct(_message.Message):
        __slots__ = ("field",)
        FIELD_FIELD_NUMBER: _ClassVar[int]
        field: _containers.RepeatedCompositeFieldContainer[ValueProto]
        def __init__(self, field: _Optional[_Iterable[_Union[ValueProto, _Mapping]]] = ...) -> None: ...
    class Datetime(_message.Message):
        __slots__ = ("bit_field_datetime_seconds", "nanos")
        BIT_FIELD_DATETIME_SECONDS_FIELD_NUMBER: _ClassVar[int]
        NANOS_FIELD_NUMBER: _ClassVar[int]
        bit_field_datetime_seconds: int
        nanos: int
        def __init__(self, bit_field_datetime_seconds: _Optional[int] = ..., nanos: _Optional[int] = ...) -> None: ...
    class Range(_message.Message):
        __slots__ = ("start", "end")
        START_FIELD_NUMBER: _ClassVar[int]
        END_FIELD_NUMBER: _ClassVar[int]
        start: ValueProto
        end: ValueProto
        def __init__(self, start: _Optional[_Union[ValueProto, _Mapping]] = ..., end: _Optional[_Union[ValueProto, _Mapping]] = ...) -> None: ...
    class MapEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: ValueProto
        value: ValueProto
        def __init__(self, key: _Optional[_Union[ValueProto, _Mapping]] = ..., value: _Optional[_Union[ValueProto, _Mapping]] = ...) -> None: ...
    class Map(_message.Message):
        __slots__ = ("entry",)
        ENTRY_FIELD_NUMBER: _ClassVar[int]
        entry: _containers.RepeatedCompositeFieldContainer[ValueProto.MapEntry]
        def __init__(self, entry: _Optional[_Iterable[_Union[ValueProto.MapEntry, _Mapping]]] = ...) -> None: ...
    class TimestampPicos(_message.Message):
        __slots__ = ("seconds", "picos")
        SECONDS_FIELD_NUMBER: _ClassVar[int]
        PICOS_FIELD_NUMBER: _ClassVar[int]
        seconds: int
        picos: int
        def __init__(self, seconds: _Optional[int] = ..., picos: _Optional[int] = ...) -> None: ...
    INT32_VALUE_FIELD_NUMBER: _ClassVar[int]
    INT64_VALUE_FIELD_NUMBER: _ClassVar[int]
    UINT32_VALUE_FIELD_NUMBER: _ClassVar[int]
    UINT64_VALUE_FIELD_NUMBER: _ClassVar[int]
    BOOL_VALUE_FIELD_NUMBER: _ClassVar[int]
    FLOAT_VALUE_FIELD_NUMBER: _ClassVar[int]
    DOUBLE_VALUE_FIELD_NUMBER: _ClassVar[int]
    STRING_VALUE_FIELD_NUMBER: _ClassVar[int]
    BYTES_VALUE_FIELD_NUMBER: _ClassVar[int]
    DATE_VALUE_FIELD_NUMBER: _ClassVar[int]
    ENUM_VALUE_FIELD_NUMBER: _ClassVar[int]
    ARRAY_VALUE_FIELD_NUMBER: _ClassVar[int]
    STRUCT_VALUE_FIELD_NUMBER: _ClassVar[int]
    PROTO_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_PICO_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_PICOS_VALUE_FIELD_NUMBER: _ClassVar[int]
    DATETIME_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME_VALUE_FIELD_NUMBER: _ClassVar[int]
    GEOGRAPHY_VALUE_FIELD_NUMBER: _ClassVar[int]
    NUMERIC_VALUE_FIELD_NUMBER: _ClassVar[int]
    BIGNUMERIC_VALUE_FIELD_NUMBER: _ClassVar[int]
    JSON_VALUE_FIELD_NUMBER: _ClassVar[int]
    INTERVAL_VALUE_FIELD_NUMBER: _ClassVar[int]
    TOKENLIST_VALUE_FIELD_NUMBER: _ClassVar[int]
    RANGE_VALUE_FIELD_NUMBER: _ClassVar[int]
    UUID_VALUE_FIELD_NUMBER: _ClassVar[int]
    MAP_VALUE_FIELD_NUMBER: _ClassVar[int]
    __VALUEPROTO__SWITCH_MUST_HAVE_A_DEFAULT_FIELD_NUMBER: _ClassVar[int]
    int32_value: int
    int64_value: int
    uint32_value: int
    uint64_value: int
    bool_value: bool
    float_value: float
    double_value: float
    string_value: str
    bytes_value: bytes
    date_value: int
    enum_value: int
    array_value: ValueProto.Array
    struct_value: ValueProto.Struct
    proto_value: bytes
    timestamp_value: _timestamp_pb2.Timestamp
    timestamp_pico_value: bytes
    timestamp_picos_value: ValueProto.TimestampPicos
    datetime_value: ValueProto.Datetime
    time_value: int
    geography_value: bytes
    numeric_value: bytes
    bignumeric_value: bytes
    json_value: str
    interval_value: bytes
    tokenlist_value: bytes
    range_value: ValueProto.Range
    uuid_value: bytes
    map_value: ValueProto.Map
    __ValueProto__switch_must_have_a_default: bool
    def __init__(self, int32_value: _Optional[int] = ..., int64_value: _Optional[int] = ..., uint32_value: _Optional[int] = ..., uint64_value: _Optional[int] = ..., bool_value: bool = ..., float_value: _Optional[float] = ..., double_value: _Optional[float] = ..., string_value: _Optional[str] = ..., bytes_value: _Optional[bytes] = ..., date_value: _Optional[int] = ..., enum_value: _Optional[int] = ..., array_value: _Optional[_Union[ValueProto.Array, _Mapping]] = ..., struct_value: _Optional[_Union[ValueProto.Struct, _Mapping]] = ..., proto_value: _Optional[bytes] = ..., timestamp_value: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., timestamp_pico_value: _Optional[bytes] = ..., timestamp_picos_value: _Optional[_Union[ValueProto.TimestampPicos, _Mapping]] = ..., datetime_value: _Optional[_Union[ValueProto.Datetime, _Mapping]] = ..., time_value: _Optional[int] = ..., geography_value: _Optional[bytes] = ..., numeric_value: _Optional[bytes] = ..., bignumeric_value: _Optional[bytes] = ..., json_value: _Optional[str] = ..., interval_value: _Optional[bytes] = ..., tokenlist_value: _Optional[bytes] = ..., range_value: _Optional[_Union[ValueProto.Range, _Mapping]] = ..., uuid_value: _Optional[bytes] = ..., map_value: _Optional[_Union[ValueProto.Map, _Mapping]] = ..., __ValueProto__switch_must_have_a_default: bool = ...) -> None: ...



import re
from enum import IntEnum


class StatusCode(IntEnum):
    """absl::StatusCode enum values.

    Matches the C++ absl::StatusCode and gRPC status codes.
    See: https://github.com/abseil/abseil-cpp/blob/master/absl/status/status.h
    """

    OK = 0
    CANCELLED = 1
    UNKNOWN = 2
    INVALID_ARGUMENT = 3
    DEADLINE_EXCEEDED = 4
    NOT_FOUND = 5
    ALREADY_EXISTS = 6
    PERMISSION_DENIED = 7
    RESOURCE_EXHAUSTED = 8
    FAILED_PRECONDITION = 9
    ABORTED = 10
    OUT_OF_RANGE = 11
    UNIMPLEMENTED = 12
    INTERNAL = 13
    UNAVAILABLE = 14
    DATA_LOSS = 15
    UNAUTHENTICATED = 16


class ZetaSQLError(Exception):
    """Base exception for all ZetaSQL errors.

    This is the parent class for both server-side errors (from gRPC/backend)
    and client-side errors (from Python API validation).

    Subclasses:
        ServerError: Errors from ZetaSQL backend (SQL analysis, execution, etc.)
        ClientError: Errors from Python API usage (parameter validation, state errors)
    """


class ServerError(ZetaSQLError):
    """Base exception for ZetaSQL backend/server errors.

    Raised when the ZetaSQL server (via gRPC) returns an error.
    Contains absl::StatusCode from the C++ backend.

    Attributes:
        code: absl::StatusCode as StatusCode enum or int
        message: Error message from ZetaSQL backend
        raw_error: Original error string from C++ ("Code: X, Message: Y")

    Example:
        try:
            response = service.prepare_query(sql=sql, catalog=catalog)
        except ServerError as e:
            if e.code == StatusCode.INVALID_ARGUMENT:
                print(f"SQL syntax error: {e.message}")
            elif e.code == StatusCode.NOT_FOUND:
                print(f"Table not found: {e.message}")
    """

    def __init__(self, code: int, message: str, raw_error: str):
        self.code = StatusCode(code) if code in StatusCode._value2member_map_ else code
        self.message = message
        self.raw_error = raw_error
        super().__init__(f"[{self.code.name if isinstance(self.code, StatusCode) else f'Code {code}'}] {message}")

    @classmethod
    def from_error_string(cls, error_str: str):
        """Parse error string from C++ format: 'Code: X, Message: Y'

        Args:
            error_str: Error string in format "Code: X, Message: Y"

        Returns:
            ServerError instance with parsed code and message
        """
        match = re.match(r"Code: (\d+), Message: (.+)", error_str)
        if match:
            code = int(match.group(1))
            message = match.group(2)
            return cls(code, message, error_str)
        # Fallback for unexpected format
        return cls(StatusCode.UNKNOWN, error_str, error_str)


class ClientError(ZetaSQLError):
    """Base exception for Python API client-side errors.

    Raised when the Python API detects invalid usage before calling the backend.
    Does not contain StatusCode (not from gRPC).

    Examples:
        - Missing required parameters
        - Mutually exclusive parameters both provided
        - Invalid state (e.g., using closed PreparedQuery)

    Example:
        try:
            query = PreparedQuery.builder().prepare()  # Missing SQL
        except ClientError as e:
            print(f"API usage error: {e}")
    """


class InvalidArgumentError(ClientError):
    """Invalid argument provided to Python API.

    Raised for parameter validation failures such as:
    - Missing required parameters
    - Mutually exclusive parameters both provided
    - Invalid parameter combinations
    - Empty or malformed input

    Example:
        try:
            query = builder.prepare()  # Missing SQL
        except InvalidArgumentError as e:
            print(f"Invalid parameters: {e}")
    """


class IllegalStateError(ClientError):
    """Operation called in illegal state.

    Raised when an operation is attempted in an invalid state:
    - Using PreparedQuery after close()
    - Calling methods on disposed objects
    - Invalid operation sequence

    Example:
        query.close()
        try:
            query.execute()  # Error: already closed
        except IllegalStateError as e:
            print(f"Invalid state: {e}")
    """



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/parser/ast_enums.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/parser/ast_enums.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1ezetasql/parser/ast_enums.proto\x12\x07zetasql\"\x9d\x02\n\x18\x41STBinaryExpressionEnums\"\x80\x02\n\x02Op\x12\x0b\n\x07NOT_SET\x10\x00\x12\x08\n\x04LIKE\x10\x01\x12\x06\n\x02IS\x10\x02\x12\x06\n\x02\x45Q\x10\x03\x12\x06\n\x02NE\x10\x04\x12\x07\n\x03NE2\x10\x05\x12\x06\n\x02GT\x10\x06\x12\x06\n\x02LT\x10\x07\x12\x06\n\x02GE\x10\x08\x12\x06\n\x02LE\x10\t\x12\x0e\n\nBITWISE_OR\x10\n\x12\x0f\n\x0b\x42ITWISE_XOR\x10\x0b\x12\x0f\n\x0b\x42ITWISE_AND\x10\x0c\x12\x08\n\x04PLUS\x10\r\x12\t\n\x05MINUS\x10\x0e\x12\x0c\n\x08MULTIPLY\x10\x0f\x12\n\n\x06\x44IVIDE\x10\x10\x12\r\n\tCONCAT_OP\x10\x11\x12\x0c\n\x08\x44ISTINCT\x10\x12\x12\x12\n\x0eIS_SOURCE_NODE\x10\x13\x12\x10\n\x0cIS_DEST_NODE\x10\x14\"_\n\x14\x41STOptionsEntryEnums\"G\n\x0c\x41ssignmentOp\x12\x0b\n\x07NOT_SET\x10\x00\x12\n\n\x06\x41SSIGN\x10\x01\x12\x0e\n\nADD_ASSIGN\x10\x02\x12\x0e\n\nSUB_ASSIGN\x10\x03\"]\n\x1a\x41STOrderingExpressionEnums\"?\n\x0cOrderingSpec\x12\x0b\n\x07NOT_SET\x10\x00\x12\x07\n\x03\x41SC\x10\x01\x12\x08\n\x04\x44\x45SC\x10\x02\x12\x0f\n\x0bUNSPECIFIED\x10\x03\"\xa5\x01\n\x0c\x41STJoinEnums\"a\n\x08JoinType\x12\x15\n\x11\x44\x45\x46\x41ULT_JOIN_TYPE\x10\x00\x12\t\n\x05\x43OMMA\x10\x01\x12\t\n\x05\x43ROSS\x10\x02\x12\x08\n\x04\x46ULL\x10\x03\x12\t\n\x05INNER\x10\x04\x12\x08\n\x04LEFT\x10\x05\x12\t\n\x05RIGHT\x10\x06\"2\n\x08JoinHint\x12\x10\n\x0cNO_JOIN_HINT\x10\x00\x12\x08\n\x04HASH\x10\x01\x12\n\n\x06LOOKUP\x10\x02\"O\n\x10\x41STSelectAsEnums\";\n\x06\x41sMode\x12\x0b\n\x07NOT_SET\x10\x00\x12\n\n\x06STRUCT\x10\x01\x12\t\n\x05VALUE\x10\x02\x12\r\n\tTYPE_NAME\x10\x03\"n\n\x14\x41STFunctionCallEnums\"V\n\x14NullHandlingModifier\x12\x19\n\x15\x44\x45\x46\x41ULT_NULL_HANDLING\x10\x00\x12\x10\n\x0cIGNORE_NULLS\x10\x01\x12\x11\n\rRESPECT_NULLS\x10\x02\"T\n\x1a\x41STExpressionSubqueryEnums\"6\n\x08Modifier\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05\x41RRAY\x10\x01\x12\n\n\x06\x45XISTS\x10\x02\x12\t\n\x05VALUE\x10\x03\"G\n\x16\x41STHavingModifierEnums\"-\n\x0cModifierKind\x12\x0b\n\x07NOT_SET\x10\x00\x12\x07\n\x03MIN\x10\x01\x12\x07\n\x03MAX\x10\x02\"\xcd\x02\n\x14\x41STSetOperationEnums\"B\n\rOperationType\x12\x0b\n\x07NOT_SET\x10\x00\x12\t\n\x05UNION\x10\x01\x12\n\n\x06\x45XCEPT\x10\x02\x12\r\n\tINTERSECT\x10\x04\"C\n\rAllOrDistinct\x12\x1b\n\x17\x41LL_OR_DISTINCT_NOT_SET\x10\x00\x12\x07\n\x03\x41LL\x10\x01\x12\x0c\n\x08\x44ISTINCT\x10\x02\"h\n\x0f\x43olumnMatchMode\x12\x0f\n\x0b\x42Y_POSITION\x10\x01\x12\x11\n\rCORRESPONDING\x10\x02\x12\x14\n\x10\x43ORRESPONDING_BY\x10\x03\x12\x0b\n\x07\x42Y_NAME\x10\x04\x12\x0e\n\nBY_NAME_ON\x10\x05\"B\n\x15\x43olumnPropagationMode\x12\n\n\x06STRICT\x10\x01\x12\t\n\x05INNER\x10\x02\x12\x08\n\x04LEFT\x10\x03\x12\x08\n\x04\x46ULL\x10\x04\"\x7f\n\x17\x41STUnaryExpressionEnums\"d\n\x02Op\x12\x0b\n\x07NOT_SET\x10\x00\x12\x07\n\x03NOT\x10\x01\x12\x0f\n\x0b\x42ITWISE_NOT\x10\x02\x12\t\n\x05MINUS\x10\x03\x12\x08\n\x04PLUS\x10\x04\x12\x0e\n\nIS_UNKNOWN\x10\x05\x12\x12\n\x0eIS_NOT_UNKNOWN\x10\x06\"7\n\x13\x41STWindowFrameEnums\" \n\tFrameUnit\x12\x08\n\x04ROWS\x10\x01\x12\t\n\x05RANGE\x10\x02\"\x98\x01\n\x17\x41STWindowFrameExprEnums\"}\n\x0c\x42oundaryType\x12\x17\n\x13UNBOUNDED_PRECEDING\x10\x01\x12\x14\n\x10OFFSET_PRECEDING\x10\x02\x12\x0f\n\x0b\x43URRENT_ROW\x10\x03\x12\x14\n\x10OFFSET_FOLLOWING\x10\x04\x12\x17\n\x13UNBOUNDED_FOLLOWING\x10\x05\"O\n\x14\x41STAnySomeAllOpEnums\"7\n\x02Op\x12\x12\n\x0ekUninitialized\x10\x01\x12\x08\n\x04kAny\x10\x02\x12\t\n\x05kSome\x10\x03\x12\x08\n\x04kAll\x10\x04\"V\n ASTTransactionReadWriteModeEnums\"2\n\x04Mode\x12\x0b\n\x07INVALID\x10\x00\x12\r\n\tREAD_ONLY\x10\x01\x12\x0e\n\nREAD_WRITE\x10\x02\">\n\x17\x41STImportStatementEnums\"#\n\nImportKind\x12\n\n\x06MODULE\x10\x01\x12\t\n\x05PROTO\x10\x02\"S\n\x15\x41STUnpivotClauseEnums\":\n\nNullFilter\x12\x10\n\x0ckUnspecified\x10\x01\x12\x0c\n\x08kInclude\x10\x02\x12\x0c\n\x08kExclude\x10\x03\"\xbe\x01\n\x17\x41STCreateStatementEnums\"B\n\x05Scope\x12\x11\n\rDEFAULT_SCOPE\x10\x00\x12\x0b\n\x07PRIVATE\x10\x01\x12\n\n\x06PUBLIC\x10\x02\x12\r\n\tTEMPORARY\x10\x03\"_\n\x0bSqlSecurity\x12\x1c\n\x18SQL_SECURITY_UNSPECIFIED\x10\x00\x12\x18\n\x14SQL_SECURITY_DEFINER\x10\x01\x12\x18\n\x14SQL_SECURITY_INVOKER\x10\x02\"^\n\x19\x41STFunctionParameterEnums\"A\n\x16ProcedureParameterMode\x12\x0b\n\x07NOT_SET\x10\x00\x12\x06\n\x02IN\x10\x01\x12\x07\n\x03OUT\x10\x02\x12\t\n\x05INOUT\x10\x03\"\xa1\x01\n\x1e\x41STTemplatedParameterTypeEnums\"\x7f\n\x11TemplatedTypeKind\x12\x11\n\rUNINITIALIZED\x10\x01\x12\x0c\n\x08\x41NY_TYPE\x10\x02\x12\r\n\tANY_PROTO\x10\x03\x12\x0c\n\x08\x41NY_ENUM\x10\x04\x12\x0e\n\nANY_STRUCT\x10\x05\x12\r\n\tANY_ARRAY\x10\x06\x12\r\n\tANY_TABLE\x10\x07\"\x89\x01\n\x1b\x41STGeneratedColumnInfoEnums\"=\n\nStoredMode\x12\x0e\n\nNON_STORED\x10\x00\x12\n\n\x06STORED\x10\x01\x12\x13\n\x0fSTORED_VOLATILE\x10\x02\"+\n\rGeneratedMode\x12\n\n\x06\x41LWAYS\x10\x00\x12\x0e\n\nBY_DEFAULT\x10\x01\"N\n\x16\x41STColumnPositionEnums\"4\n\x14RelativePositionType\x12\r\n\tPRECEDING\x10\x01\x12\r\n\tFOLLOWING\x10\x02\"\xdc\x01\n\x17\x41STInsertStatementEnums\"C\n\nInsertMode\x12\x10\n\x0c\x44\x45\x46\x41ULT_MODE\x10\x00\x12\x0b\n\x07REPLACE\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\n\n\x06IGNORE\x10\x03\"|\n\rParseProgress\x12\x0c\n\x08kInitial\x10\x01\x12\x1e\n\x1akSeenOrIgnoreReplaceUpdate\x10\x02\x12\x13\n\x0fkSeenTargetPath\x10\x03\x12\x13\n\x0fkSeenColumnList\x10\x04\x12\x13\n\x0fkSeenValuesList\x10\x05\"R\n\x18\x41STOnConflictClauseEnums\"6\n\x0e\x43onflictAction\x12\x0b\n\x07NOT_SET\x10\x00\x12\x0b\n\x07NOTHING\x10\x01\x12\n\n\x06UPDATE\x10\x02\"T\n\x13\x41STMergeActionEnums\"=\n\nActionType\x12\x0b\n\x07NOT_SET\x10\x00\x12\n\n\x06INSERT\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\n\n\x06\x44\x45LETE\x10\x03\"v\n\x17\x41STMergeWhenClauseEnums\"[\n\tMatchType\x12\x0b\n\x07NOT_SET\x10\x00\x12\x0b\n\x07MATCHED\x10\x01\x12\x19\n\x15NOT_MATCHED_BY_SOURCE\x10\x02\x12\x19\n\x15NOT_MATCHED_BY_TARGET\x10\x03\"N\n\x17\x41STFilterFieldsArgEnums\"3\n\nFilterType\x12\x0b\n\x07NOT_SET\x10\x00\x12\x0b\n\x07INCLUDE\x10\x01\x12\x0b\n\x07\x45XCLUDE\x10\x02\"@\n\x12\x41STSampleSizeEnums\"*\n\x04Unit\x12\x0b\n\x07NOT_SET\x10\x00\x12\x08\n\x04ROWS\x10\x01\x12\x0b\n\x07PERCENT\x10\x02\"]\n\x19\x41STForeignKeyActionsEnums\"@\n\x06\x41\x63tion\x12\r\n\tNO_ACTION\x10\x00\x12\x0c\n\x08RESTRICT\x10\x01\x12\x0b\n\x07\x43\x41SCADE\x10\x02\x12\x0c\n\x08SET_NULL\x10\x03\"N\n\x1b\x41STForeignKeyReferenceEnums\"/\n\x05Match\x12\n\n\x06SIMPLE\x10\x01\x12\x08\n\x04\x46ULL\x10\x02\x12\x10\n\x0cNOT_DISTINCT\x10\x03\"i\n\x1e\x41STBreakContinueStatementEnums\"G\n\x14\x42reakContinueKeyword\x12\t\n\x05\x42REAK\x10\x01\x12\t\n\x05LEAVE\x10\x02\x12\x0c\n\x08\x43ONTINUE\x10\x03\x12\x0b\n\x07ITERATE\x10\x04\"Y\n\x15\x41STDropStatementEnums\"@\n\x08\x44ropMode\x12\x19\n\x15\x44ROP_MODE_UNSPECIFIED\x10\x00\x12\x0c\n\x08RESTRICT\x10\x01\x12\x0b\n\x07\x43\x41SCADE\x10\x02\"\xa5\x01\n\x1e\x41STCreateFunctionStmtBaseEnums\"\x82\x01\n\x10\x44\x65terminismLevel\x12\x1b\n\x17\x44\x45TERMINISM_UNSPECIFIED\x10\x00\x12\x11\n\rDETERMINISTIC\x10\x01\x12\x15\n\x11NOT_DETERMINISTIC\x10\x02\x12\r\n\tIMMUTABLE\x10\x03\x12\n\n\x06STABLE\x10\x04\x12\x0c\n\x08VOLATILE\x10\x05\"W\n\x1c\x41STAuxLoadDataStatementEnums\"7\n\rInsertionMode\x12\x0b\n\x07NOT_SET\x10\x00\x12\n\n\x06\x41PPEND\x10\x01\x12\r\n\tOVERWRITE\x10\x02\"M\n\x1f\x41STSpannerInterleaveClauseEnums\"*\n\x04Type\x12\x0b\n\x07NOT_SET\x10\x00\x12\x06\n\x02IN\x10\x01\x12\r\n\tIN_PARENT\x10\x02\"\x87\x01\n\x1c\x41STAfterMatchSkipClauseEnums\"g\n\x18\x41\x66terMatchSkipTargetType\x12\'\n#AFTER_MATCH_SKIP_TARGET_UNSPECIFIED\x10\x00\x12\x11\n\rPAST_LAST_ROW\x10\x01\x12\x0f\n\x0bTO_NEXT_ROW\x10\x02\"P\n\x18\x41STRowPatternAnchorEnums\"4\n\x06\x41nchor\x12\x16\n\x12\x41NCHOR_UNSPECIFIED\x10\x00\x12\t\n\x05START\x10\x01\x12\x07\n\x03\x45ND\x10\x02\"\x83\x01\n\x1b\x41STRowPatternOperationEnums\"d\n\rOperationType\x12\x1e\n\x1aOPERATION_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06\x43ONCAT\x10\x01\x12\r\n\tALTERNATE\x10\x02\x12\x0b\n\x07PERMUTE\x10\x03\x12\x0b\n\x07\x45XCLUDE\x10\x04\"c\n\x18\x41STSymbolQuantifierEnums\"G\n\x06Symbol\x12\x16\n\x12SYMBOL_UNSPECIFIED\x10\x00\x12\x11\n\rQUESTION_MARK\x10\x01\x12\x08\n\x04PLUS\x10\x02\x12\x08\n\x04STAR\x10\x03\"x\n\x1f\x41STGraphNodeTableReferenceEnums\"U\n\x11NodeReferenceType\x12#\n\x1fNODE_REFERENCE_TYPE_UNSPECIFIED\x10\x00\x12\n\n\x06SOURCE\x10\x01\x12\x0f\n\x0b\x44\x45STINATION\x10\x02\"h\n\x1b\x41STGraphLabelOperationEnums\"I\n\rOperationType\x12\x1e\n\x1aOPERATION_TYPE_UNSPECIFIED\x10\x00\x12\x07\n\x03NOT\x10\x01\x12\x07\n\x03\x41ND\x10\x02\x12\x06\n\x02OR\x10\x03\"i\n\x18\x41STGraphEdgePatternEnums\"M\n\x0f\x45\x64geOrientation\x12\x1c\n\x18\x45\x44GE_ORIENTATION_NOT_SET\x10\x00\x12\x07\n\x03\x41NY\x10\x01\x12\x08\n\x04LEFT\x10\x02\x12\t\n\x05RIGHT\x10\x03\"l\n\x15\x41STGraphPathModeEnums\"S\n\x08PathMode\x12\x19\n\x15PATH_MODE_UNSPECIFIED\x10\x00\x12\x08\n\x04WALK\x10\x01\x12\t\n\x05TRAIL\x10\x02\x12\n\n\x06SIMPLE\x10\x03\x12\x0b\n\x07\x41\x43YCLIC\x10\x04\"\xb3\x01\n\x1d\x41STGraphPathSearchPrefixEnums\"\x91\x01\n\x14PathSearchPrefixType\x12\'\n#PATH_SEARCH_PREFIX_TYPE_UNSPECIFIED\x10\x00\x12\x07\n\x03\x41NY\x10\x01\x12\x0c\n\x08SHORTEST\x10\x02\x12\x07\n\x03\x41LL\x10\x03\x12\x10\n\x0c\x41LL_SHORTEST\x10\x04\x12\x0c\n\x08\x43HEAPEST\x10\x05\x12\x10\n\x0c\x41LL_CHEAPEST\x10\x06\"?\n\x10\x41STLockModeEnums\"+\n\x10LockStrengthSpec\x12\x0b\n\x07NOT_SET\x10\x00\x12\n\n\x06UPDATE\x10\x01\"n\n\x1c\x41STBracedConstructorLhsEnums\"N\n\tOperation\x12\x11\n\rUPDATE_SINGLE\x10\x00\x12\x0f\n\x0bUPDATE_MANY\x10\x01\x12\x1d\n\x19UPDATE_SINGLE_NO_CREATION\x10\x02\"a\n\x1b\x41STAlterIndexStatementEnums\"B\n\tIndexType\x12\x11\n\rINDEX_DEFAULT\x10\x00\x12\x10\n\x0cINDEX_SEARCH\x10\x01\x12\x10\n\x0cINDEX_VECTOR\x10\x02*\xad\x03\n\x10SchemaObjectKind\x12=\n0__SchemaObjectKind__switch_must_have_a_default__\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x1c\n\x18kInvalidSchemaObjectKind\x10\x01\x12\x16\n\x12kAggregateFunction\x10\x02\x12\x0f\n\x0bkApproxView\x10\x11\x12\x0f\n\x0bkConnection\x10\x13\x12\r\n\tkConstant\x10\x03\x12\r\n\tkDatabase\x10\x04\x12\x12\n\x0ekExternalTable\x10\x05\x12\r\n\tkFunction\x10\x06\x12\n\n\x06kIndex\x10\x07\x12\x15\n\x11kMaterializedView\x10\x08\x12\n\n\x06kModel\x10\t\x12\x0e\n\nkProcedure\x10\n\x12\x0b\n\x07kSchema\x10\x0b\x12\n\n\x06kTable\x10\x0c\x12\x12\n\x0ekTableFunction\x10\r\x12\t\n\x05kView\x10\x0e\x12\x12\n\x0ekSnapshotTable\x10\x0f\x12\x12\n\x0ekPropertyGraph\x10\x10\x12\x13\n\x0fkExternalSchema\x10\x12\x12\r\n\tkSequence\x10\x14\x42\x1d\n\x19\x63om.google.zetasql.parserP\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.parser.ast_enums_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\031com.google.zetasql.parserP\001'
  _globals['_SCHEMAOBJECTKIND']._serialized_start=5438
  _globals['_SCHEMAOBJECTKIND']._serialized_end=5867
  _globals['_ASTBINARYEXPRESSIONENUMS']._serialized_start=44
  _globals['_ASTBINARYEXPRESSIONENUMS']._serialized_end=329
  _globals['_ASTBINARYEXPRESSIONENUMS_OP']._serialized_start=73
  _globals['_ASTBINARYEXPRESSIONENUMS_OP']._serialized_end=329
  _globals['_ASTOPTIONSENTRYENUMS']._serialized_start=331
  _globals['_ASTOPTIONSENTRYENUMS']._serialized_end=426
  _globals['_ASTOPTIONSENTRYENUMS_ASSIGNMENTOP']._serialized_start=355
  _globals['_ASTOPTIONSENTRYENUMS_ASSIGNMENTOP']._serialized_end=426
  _globals['_ASTORDERINGEXPRESSIONENUMS']._serialized_start=428
  _globals['_ASTORDERINGEXPRESSIONENUMS']._serialized_end=521
  _globals['_ASTORDERINGEXPRESSIONENUMS_ORDERINGSPEC']._serialized_start=458
  _globals['_ASTORDERINGEXPRESSIONENUMS_ORDERINGSPEC']._serialized_end=521
  _globals['_ASTJOINENUMS']._serialized_start=524
  _globals['_ASTJOINENUMS']._serialized_end=689
  _globals['_ASTJOINENUMS_JOINTYPE']._serialized_start=540
  _globals['_ASTJOINENUMS_JOINTYPE']._serialized_end=637
  _globals['_ASTJOINENUMS_JOINHINT']._serialized_start=639
  _globals['_ASTJOINENUMS_JOINHINT']._serialized_end=689
  _globals['_ASTSELECTASENUMS']._serialized_start=691
  _globals['_ASTSELECTASENUMS']._serialized_end=770
  _globals['_ASTSELECTASENUMS_ASMODE']._serialized_start=711
  _globals['_ASTSELECTASENUMS_ASMODE']._serialized_end=770
  _globals['_ASTFUNCTIONCALLENUMS']._serialized_start=772
  _globals['_ASTFUNCTIONCALLENUMS']._serialized_end=882
  _globals['_ASTFUNCTIONCALLENUMS_NULLHANDLINGMODIFIER']._serialized_start=796
  _globals['_ASTFUNCTIONCALLENUMS_NULLHANDLINGMODIFIER']._serialized_end=882
  _globals['_ASTEXPRESSIONSUBQUERYENUMS']._serialized_start=884
  _globals['_ASTEXPRESSIONSUBQUERYENUMS']._serialized_end=968
  _globals['_ASTEXPRESSIONSUBQUERYENUMS_MODIFIER']._serialized_start=914
  _globals['_ASTEXPRESSIONSUBQUERYENUMS_MODIFIER']._serialized_end=968
  _globals['_ASTHAVINGMODIFIERENUMS']._serialized_start=970
  _globals['_ASTHAVINGMODIFIERENUMS']._serialized_end=1041
  _globals['_ASTHAVINGMODIFIERENUMS_MODIFIERKIND']._serialized_start=996
  _globals['_ASTHAVINGMODIFIERENUMS_MODIFIERKIND']._serialized_end=1041
  _globals['_ASTSETOPERATIONENUMS']._serialized_start=1044
  _globals['_ASTSETOPERATIONENUMS']._serialized_end=1377
  _globals['_ASTSETOPERATIONENUMS_OPERATIONTYPE']._serialized_start=1068
  _globals['_ASTSETOPERATIONENUMS_OPERATIONTYPE']._serialized_end=1134
  _globals['_ASTSETOPERATIONENUMS_ALLORDISTINCT']._serialized_start=1136
  _globals['_ASTSETOPERATIONENUMS_ALLORDISTINCT']._serialized_end=1203
  _globals['_ASTSETOPERATIONENUMS_COLUMNMATCHMODE']._serialized_start=1205
  _globals['_ASTSETOPERATIONENUMS_COLUMNMATCHMODE']._serialized_end=1309
  _globals['_ASTSETOPERATIONENUMS_COLUMNPROPAGATIONMODE']._serialized_start=1311
  _globals['_ASTSETOPERATIONENUMS_COLUMNPROPAGATIONMODE']._serialized_end=1377
  _globals['_ASTUNARYEXPRESSIONENUMS']._serialized_start=1379
  _globals['_ASTUNARYEXPRESSIONENUMS']._serialized_end=1506
  _globals['_ASTUNARYEXPRESSIONENUMS_OP']._serialized_start=1406
  _globals['_ASTUNARYEXPRESSIONENUMS_OP']._serialized_end=1506
  _globals['_ASTWINDOWFRAMEENUMS']._serialized_start=1508
  _globals['_ASTWINDOWFRAMEENUMS']._serialized_end=1563
  _globals['_ASTWINDOWFRAMEENUMS_FRAMEUNIT']._serialized_start=1531
  _globals['_ASTWINDOWFRAMEENUMS_FRAMEUNIT']._serialized_end=1563
  _globals['_ASTWINDOWFRAMEEXPRENUMS']._serialized_start=1566
  _globals['_ASTWINDOWFRAMEEXPRENUMS']._serialized_end=1718
  _globals['_ASTWINDOWFRAMEEXPRENUMS_BOUNDARYTYPE']._serialized_start=1593
  _globals['_ASTWINDOWFRAMEEXPRENUMS_BOUNDARYTYPE']._serialized_end=1718
  _globals['_ASTANYSOMEALLOPENUMS']._serialized_start=1720
  _globals['_ASTANYSOMEALLOPENUMS']._serialized_end=1799
  _globals['_ASTANYSOMEALLOPENUMS_OP']._serialized_start=1744
  _globals['_ASTANYSOMEALLOPENUMS_OP']._serialized_end=1799
  _globals['_ASTTRANSACTIONREADWRITEMODEENUMS']._serialized_start=1801
  _globals['_ASTTRANSACTIONREADWRITEMODEENUMS']._serialized_end=1887
  _globals['_ASTTRANSACTIONREADWRITEMODEENUMS_MODE']._serialized_start=1837
  _globals['_ASTTRANSACTIONREADWRITEMODEENUMS_MODE']._serialized_end=1887
  _globals['_ASTIMPORTSTATEMENTENUMS']._serialized_start=1889
  _globals['_ASTIMPORTSTATEMENTENUMS']._serialized_end=1951
  _globals['_ASTIMPORTSTATEMENTENUMS_IMPORTKIND']._serialized_start=1916
  _globals['_ASTIMPORTSTATEMENTENUMS_IMPORTKIND']._serialized_end=1951
  _globals['_ASTUNPIVOTCLAUSEENUMS']._serialized_start=1953
  _globals['_ASTUNPIVOTCLAUSEENUMS']._serialized_end=2036
  _globals['_ASTUNPIVOTCLAUSEENUMS_NULLFILTER']._serialized_start=1978
  _globals['_ASTUNPIVOTCLAUSEENUMS_NULLFILTER']._serialized_end=2036
  _globals['_ASTCREATESTATEMENTENUMS']._serialized_start=2039
  _globals['_ASTCREATESTATEMENTENUMS']._serialized_end=2229
  _globals['_ASTCREATESTATEMENTENUMS_SCOPE']._serialized_start=2066
  _globals['_ASTCREATESTATEMENTENUMS_SCOPE']._serialized_end=2132
  _globals['_ASTCREATESTATEMENTENUMS_SQLSECURITY']._serialized_start=2134
  _globals['_ASTCREATESTATEMENTENUMS_SQLSECURITY']._serialized_end=2229
  _globals['_ASTFUNCTIONPARAMETERENUMS']._serialized_start=2231
  _globals['_ASTFUNCTIONPARAMETERENUMS']._serialized_end=2325
  _globals['_ASTFUNCTIONPARAMETERENUMS_PROCEDUREPARAMETERMODE']._serialized_start=2260
  _globals['_ASTFUNCTIONPARAMETERENUMS_PROCEDUREPARAMETERMODE']._serialized_end=2325
  _globals['_ASTTEMPLATEDPARAMETERTYPEENUMS']._serialized_start=2328
  _globals['_ASTTEMPLATEDPARAMETERTYPEENUMS']._serialized_end=2489
  _globals['_ASTTEMPLATEDPARAMETERTYPEENUMS_TEMPLATEDTYPEKIND']._serialized_start=2362
  _globals['_ASTTEMPLATEDPARAMETERTYPEENUMS_TEMPLATEDTYPEKIND']._serialized_end=2489
  _globals['_ASTGENERATEDCOLUMNINFOENUMS']._serialized_start=2492
  _globals['_ASTGENERATEDCOLUMNINFOENUMS']._serialized_end=2629
  _globals['_ASTGENERATEDCOLUMNINFOENUMS_STOREDMODE']._serialized_start=2523
  _globals['_ASTGENERATEDCOLUMNINFOENUMS_STOREDMODE']._serialized_end=2584
  _globals['_ASTGENERATEDCOLUMNINFOENUMS_GENERATEDMODE']._serialized_start=2586
  _globals['_ASTGENERATEDCOLUMNINFOENUMS_GENERATEDMODE']._serialized_end=2629
  _globals['_ASTCOLUMNPOSITIONENUMS']._serialized_start=2631
  _globals['_ASTCOLUMNPOSITIONENUMS']._serialized_end=2709
  _globals['_ASTCOLUMNPOSITIONENUMS_RELATIVEPOSITIONTYPE']._serialized_start=2657
  _globals['_ASTCOLUMNPOSITIONENUMS_RELATIVEPOSITIONTYPE']._serialized_end=2709
  _globals['_ASTINSERTSTATEMENTENUMS']._serialized_start=2712
  _globals['_ASTINSERTSTATEMENTENUMS']._serialized_end=2932
  _globals['_ASTINSERTSTATEMENTENUMS_INSERTMODE']._serialized_start=2739
  _globals['_ASTINSERTSTATEMENTENUMS_INSERTMODE']._serialized_end=2806
  _globals['_ASTINSERTSTATEMENTENUMS_PARSEPROGRESS']._serialized_start=2808
  _globals['_ASTINSERTSTATEMENTENUMS_PARSEPROGRESS']._serialized_end=2932
  _globals['_ASTONCONFLICTCLAUSEENUMS']._serialized_start=2934
  _globals['_ASTONCONFLICTCLAUSEENUMS']._serialized_end=3016
  _globals['_ASTONCONFLICTCLAUSEENUMS_CONFLICTACTION']._serialized_start=2962
  _globals['_ASTONCONFLICTCLAUSEENUMS_CONFLICTACTION']._serialized_end=3016
  _globals['_ASTMERGEACTIONENUMS']._serialized_start=3018
  _globals['_ASTMERGEACTIONENUMS']._serialized_end=3102
  _globals['_ASTMERGEACTIONENUMS_ACTIONTYPE']._serialized_start=3041
  _globals['_ASTMERGEACTIONENUMS_ACTIONTYPE']._serialized_end=3102
  _globals['_ASTMERGEWHENCLAUSEENUMS']._serialized_start=3104
  _globals['_ASTMERGEWHENCLAUSEENUMS']._serialized_end=3222
  _globals['_ASTMERGEWHENCLAUSEENUMS_MATCHTYPE']._serialized_start=3131
  _globals['_ASTMERGEWHENCLAUSEENUMS_MATCHTYPE']._serialized_end=3222
  _globals['_ASTFILTERFIELDSARGENUMS']._serialized_start=3224
  _globals['_ASTFILTERFIELDSARGENUMS']._serialized_end=3302
  _globals['_ASTFILTERFIELDSARGENUMS_FILTERTYPE']._serialized_start=3251
  _globals['_ASTFILTERFIELDSARGENUMS_FILTERTYPE']._serialized_end=3302
  _globals['_ASTSAMPLESIZEENUMS']._serialized_start=3304
  _globals['_ASTSAMPLESIZEENUMS']._serialized_end=3368
  _globals['_ASTSAMPLESIZEENUMS_UNIT']._serialized_start=3326
  _globals['_ASTSAMPLESIZEENUMS_UNIT']._serialized_end=3368
  _globals['_ASTFOREIGNKEYACTIONSENUMS']._serialized_start=3370
  _globals['_ASTFOREIGNKEYACTIONSENUMS']._serialized_end=3463
  _globals['_ASTFOREIGNKEYACTIONSENUMS_ACTION']._serialized_start=3399
  _globals['_ASTFOREIGNKEYACTIONSENUMS_ACTION']._serialized_end=3463
  _globals['_ASTFOREIGNKEYREFERENCEENUMS']._serialized_start=3465
  _globals['_ASTFOREIGNKEYREFERENCEENUMS']._serialized_end=3543
  _globals['_ASTFOREIGNKEYREFERENCEENUMS_MATCH']._serialized_start=3496
  _globals['_ASTFOREIGNKEYREFERENCEENUMS_MATCH']._serialized_end=3543
  _globals['_ASTBREAKCONTINUESTATEMENTENUMS']._serialized_start=3545
  _globals['_ASTBREAKCONTINUESTATEMENTENUMS']._serialized_end=3650
  _globals['_ASTBREAKCONTINUESTATEMENTENUMS_BREAKCONTINUEKEYWORD']._serialized_start=3579
  _globals['_ASTBREAKCONTINUESTATEMENTENUMS_BREAKCONTINUEKEYWORD']._serialized_end=3650
  _globals['_ASTDROPSTATEMENTENUMS']._serialized_start=3652
  _globals['_ASTDROPSTATEMENTENUMS']._serialized_end=3741
  _globals['_ASTDROPSTATEMENTENUMS_DROPMODE']._serialized_start=3677
  _globals['_ASTDROPSTATEMENTENUMS_DROPMODE']._serialized_end=3741
  _globals['_ASTCREATEFUNCTIONSTMTBASEENUMS']._serialized_start=3744
  _globals['_ASTCREATEFUNCTIONSTMTBASEENUMS']._serialized_end=3909
  _globals['_ASTCREATEFUNCTIONSTMTBASEENUMS_DETERMINISMLEVEL']._serialized_start=3779
  _globals['_ASTCREATEFUNCTIONSTMTBASEENUMS_DETERMINISMLEVEL']._serialized_end=3909
  _globals['_ASTAUXLOADDATASTATEMENTENUMS']._serialized_start=3911
  _globals['_ASTAUXLOADDATASTATEMENTENUMS']._serialized_end=3998
  _globals['_ASTAUXLOADDATASTATEMENTENUMS_INSERTIONMODE']._serialized_start=3943
  _globals['_ASTAUXLOADDATASTATEMENTENUMS_INSERTIONMODE']._serialized_end=3998
  _globals['_ASTSPANNERINTERLEAVECLAUSEENUMS']._serialized_start=4000
  _globals['_ASTSPANNERINTERLEAVECLAUSEENUMS']._serialized_end=4077
  _globals['_ASTSPANNERINTERLEAVECLAUSEENUMS_TYPE']._serialized_start=4035
  _globals['_ASTSPANNERINTERLEAVECLAUSEENUMS_TYPE']._serialized_end=4077
  _globals['_ASTAFTERMATCHSKIPCLAUSEENUMS']._serialized_start=4080
  _globals['_ASTAFTERMATCHSKIPCLAUSEENUMS']._serialized_end=4215
  _globals['_ASTAFTERMATCHSKIPCLAUSEENUMS_AFTERMATCHSKIPTARGETTYPE']._serialized_start=4112
  _globals['_ASTAFTERMATCHSKIPCLAUSEENUMS_AFTERMATCHSKIPTARGETTYPE']._serialized_end=4215
  _globals['_ASTROWPATTERNANCHORENUMS']._serialized_start=4217
  _globals['_ASTROWPATTERNANCHORENUMS']._serialized_end=4297
  _globals['_ASTROWPATTERNANCHORENUMS_ANCHOR']._serialized_start=4245
  _globals['_ASTROWPATTERNANCHORENUMS_ANCHOR']._serialized_end=4297
  _globals['_ASTROWPATTERNOPERATIONENUMS']._serialized_start=4300
  _globals['_ASTROWPATTERNOPERATIONENUMS']._serialized_end=4431
  _globals['_ASTROWPATTERNOPERATIONENUMS_OPERATIONTYPE']._serialized_start=4331
  _globals['_ASTROWPATTERNOPERATIONENUMS_OPERATIONTYPE']._serialized_end=4431
  _globals['_ASTSYMBOLQUANTIFIERENUMS']._serialized_start=4433
  _globals['_ASTSYMBOLQUANTIFIERENUMS']._serialized_end=4532
  _globals['_ASTSYMBOLQUANTIFIERENUMS_SYMBOL']._serialized_start=4461
  _globals['_ASTSYMBOLQUANTIFIERENUMS_SYMBOL']._serialized_end=4532
  _globals['_ASTGRAPHNODETABLEREFERENCEENUMS']._serialized_start=4534
  _globals['_ASTGRAPHNODETABLEREFERENCEENUMS']._serialized_end=4654
  _globals['_ASTGRAPHNODETABLEREFERENCEENUMS_NODEREFERENCETYPE']._serialized_start=4569
  _globals['_ASTGRAPHNODETABLEREFERENCEENUMS_NODEREFERENCETYPE']._serialized_end=4654
  _globals['_ASTGRAPHLABELOPERATIONENUMS']._serialized_start=4656
  _globals['_ASTGRAPHLABELOPERATIONENUMS']._serialized_end=4760
  _globals['_ASTGRAPHLABELOPERATIONENUMS_OPERATIONTYPE']._serialized_start=4687
  _globals['_ASTGRAPHLABELOPERATIONENUMS_OPERATIONTYPE']._serialized_end=4760
  _globals['_ASTGRAPHEDGEPATTERNENUMS']._serialized_start=4762
  _globals['_ASTGRAPHEDGEPATTERNENUMS']._serialized_end=4867
  _globals['_ASTGRAPHEDGEPATTERNENUMS_EDGEORIENTATION']._serialized_start=4790
  _globals['_ASTGRAPHEDGEPATTERNENUMS_EDGEORIENTATION']._serialized_end=4867
  _globals['_ASTGRAPHPATHMODEENUMS']._serialized_start=4869
  _globals['_ASTGRAPHPATHMODEENUMS']._serialized_end=4977
  _globals['_ASTGRAPHPATHMODEENUMS_PATHMODE']._serialized_start=4894
  _globals['_ASTGRAPHPATHMODEENUMS_PATHMODE']._serialized_end=4977
  _globals['_ASTGRAPHPATHSEARCHPREFIXENUMS']._serialized_start=4980
  _globals['_ASTGRAPHPATHSEARCHPREFIXENUMS']._serialized_end=5159
  _globals['_ASTGRAPHPATHSEARCHPREFIXENUMS_PATHSEARCHPREFIXTYPE']._serialized_start=5014
  _globals['_ASTGRAPHPATHSEARCHPREFIXENUMS_PATHSEARCHPREFIXTYPE']._serialized_end=5159
  _globals['_ASTLOCKMODEENUMS']._serialized_start=5161
  _globals['_ASTLOCKMODEENUMS']._serialized_end=5224
  _globals['_ASTLOCKMODEENUMS_LOCKSTRENGTHSPEC']._serialized_start=5181
  _globals['_ASTLOCKMODEENUMS_LOCKSTRENGTHSPEC']._serialized_end=5224
  _globals['_ASTBRACEDCONSTRUCTORLHSENUMS']._serialized_start=5226
  _globals['_ASTBRACEDCONSTRUCTORLHSENUMS']._serialized_end=5336
  _globals['_ASTBRACEDCONSTRUCTORLHSENUMS_OPERATION']._serialized_start=5258
  _globals['_ASTBRACEDCONSTRUCTORLHSENUMS_OPERATION']._serialized_end=5336
  _globals['_ASTALTERINDEXSTATEMENTENUMS']._serialized_start=5338
  _globals['_ASTALTERINDEXSTATEMENTENUMS']._serialized_end=5435
  _globals['_ASTALTERINDEXSTATEMENTENUMS_INDEXTYPE']._serialized_start=5369
  _globals['_ASTALTERINDEXSTATEMENTENUMS_INDEXTYPE']._serialized_end=5435
# @@protoc_insertion_point(module_scope)



"""Parser helper class for SQL parsing operations.

Provides Java-style Parser with both instance and static methods
for simplified SQL parsing (syntax analysis only, no semantic analysis).

Parser returns parse tree AST (ASTStatement, ASTScript) without type checking
or name resolution, making it faster than Analyzer and suitable for:
- Syntax validation
- AST inspection and traversal
- Quick statement extraction

Use with ASTNodeVisitor for AST traversal.
"""

from __future__ import annotations

import zetasql.types
from zetasql.core.local_service import ZetaSqlLocalService


class Parser:
    """Helper class for SQL parsing operations (syntax only).

    Equivalent to Java's Parser class with static and instance methods.
    Performs syntax parsing only - returns parse tree without semantic analysis.
    Does not require a catalog.

    Unlike Analyzer which returns ResolvedStatement (with semantic analysis),
    Parser returns ASTStatement (parse tree only).

    Example (instance usage):
        >>> parser = Parser(options)
        >>> stmt = parser.parse_statement("SELECT * FROM Orders")
        >>> script = parser.parse_script("SELECT 1; SELECT 2;")

    Example (static usage):
        >>> stmt = Parser.parse_statement_static("SELECT 1")
        >>> script = Parser.parse_script_static("SELECT 1; SELECT 2;")

    Example (with ASTNodeVisitor):
        >>> from zetasql.api import Parser, ASTNodeVisitor
        >>> stmt = Parser.parse_statement_static("SELECT * FROM Orders")
        >>> visitor = MyCustomVisitor()
        >>> visitor.visit(stmt)
    """

    def __init__(
        self,
        options: zetasql.types.LanguageOptions | None = None,
        service: ZetaSqlLocalService | None = None,
    ):
        """Initialize Parser with language options.

        Args:
            options: Language options for parsing (optional, uses defaults if not provided)
            service: Optional LocalService instance (uses singleton if not provided)

        Example:
            >>> options = LanguageOptions()
            >>> options.set_enabled_language_features([...])
            >>> parser = Parser(options)
        """
        self.options = options
        self.service = service or ZetaSqlLocalService.get_instance()

    def parse_statement(self, sql: str) -> zetasql.types.ASTStatement:
        """Parse SQL statement and return AST.

        Args:
            sql: SQL statement to parse

        Returns:
            Parse tree AST (ASTStatement or subclass)

        Raises:
            ZetaSQLError: If parsing fails (syntax error)

        Example:
            >>> stmt = parser.parse_statement("SELECT id FROM Orders WHERE price > 100")
            >>> print(type(stmt).__name__)  # ASTQueryStatement
        """
        response = self.service.parse(sql_statement=sql, options=self.options)
        return response.parsed_statement

    def parse_script(self, sql: str) -> zetasql.types.ASTScript:
        """Parse SQL script and return AST.

        Parses multiple statements as a script. Allows procedural constructs
        like BEGIN/END blocks when language options permit.

        Args:
            sql: SQL script to parse (can contain multiple statements)

        Returns:
            Parse tree for script (ASTScript)

        Raises:
            ZetaSQLError: If parsing fails

        Example:
            >>> script = parser.parse_script("SELECT 1; SELECT 2; SELECT 3;")
            >>> print(len(script.statement_list_node.statement_list))  # 3
        """
        response = self.service.parse(sql_statement=sql, options=self.options, allow_script=True)
        return response.parsed_script

    def parse_next_statement(
        self,
        location: zetasql.types.ParseResumeLocation,
    ) -> zetasql.types.ASTStatement | None:
        """Parse next statement from location, updating byte_position.

        Args:
            location: ParseResumeLocation that tracks position in the input.
                     Its byte_position will be updated to point after the parsed statement.

        Returns:
            Parsed statement AST, or None if no more statements

        Side effects:
            Updates location.byte_position to point after parsed statement

        Example:
            >>> location = ParseResumeLocation(input=script, byte_position=0)
            >>> stmt1 = parser.parse_next_statement(location)
            >>> stmt2 = parser.parse_next_statement(location)
        """
        # Check if we've reached the end of input
        if location.byte_position >= len(location.input):
            return None

        response = self.service.parse(
            sql_statement=location.input,
            parse_resume_location=location,
            options=self.options,
        )

        if hasattr(response, "resume_byte_position") and response.resume_byte_position is not None:
            location.byte_position = response.resume_byte_position

        return response.parsed_statement if response.parsed_statement else None

    def parse_next_script_statement(
        self,
        location: zetasql.types.ParseResumeLocation,
    ) -> zetasql.types.ASTStatement | None:
        """Parse next script statement (allows procedural constructs).

        Like parse_next_statement but allows script-specific constructs
        like BEGIN/END, IF, WHILE when language options permit.

        Args:
            location: ParseResumeLocation that tracks position.
                     Its byte_position will be updated.

        Returns:
            Parsed statement AST, or None if no more statements

        Side effects:
            Updates location.byte_position

        Example:
            >>> location = ParseResumeLocation(input=script, byte_position=0)
            >>> stmt = parser.parse_next_script_statement(location)
        """
        # Check if we've reached the end of input
        if location.byte_position >= len(location.input):
            return None

        response = self.service.parse(
            sql_statement=location.input,
            parse_resume_location=location,
            options=self.options,
            allow_script=True,
        )

        if hasattr(response, "resume_byte_position") and response.resume_byte_position is not None:
            location.byte_position = response.resume_byte_position

        return response.parsed_statement if response.parsed_statement else None

    @staticmethod
    def parse_statement_static(
        sql: str,
        options: zetasql.types.LanguageOptions | None = None,
    ) -> zetasql.types.ASTStatement:
        """Static method for one-off statement parsing.

        Convenient for single-use parsing without creating a Parser instance.

        Args:
            sql: SQL statement to parse
            options: Optional language options

        Returns:
            Parse tree AST

        Raises:
            ZetaSQLError: If parsing fails

        Example:
            >>> stmt = Parser.parse_statement_static("SELECT * FROM Orders")
            >>> print(type(stmt).__name__)  # ASTQueryStatement
        """
        service = ZetaSqlLocalService.get_instance()
        response = service.parse(sql_statement=sql, options=options)
        return response.parsed_statement

    @staticmethod
    def parse_script_static(
        sql: str,
        options: zetasql.types.LanguageOptions | None = None,
    ) -> zetasql.types.ASTScript:
        """Static method for one-off script parsing.

        Args:
            sql: SQL script to parse
            options: Optional language options

        Returns:
            Parse tree for script (ASTScript)

        Raises:
            ZetaSQLError: If parsing fails

        Example:
            >>> script = Parser.parse_script_static("SELECT 1; SELECT 2;")
            >>> print(len(script.statement_list_node.statement_list))  # 2
        """
        service = ZetaSqlLocalService.get_instance()
        response = service.parse(sql_statement=sql, options=options, allow_script=True)
        return response.parsed_script

    @staticmethod
    def parse_next_statement_static(
        location: zetasql.types.ParseResumeLocation,
        options: zetasql.types.LanguageOptions | None = None,
    ) -> zetasql.types.ASTStatement | None:
        """Static method for parsing next statement in sequence.

        Args:
            location: ParseResumeLocation that tracks position.
                     Its byte_position will be updated.
            options: Optional language options

        Returns:
            Parsed statement AST, or None if no more statements

        Example:
            >>> location = ParseResumeLocation(input=script, byte_position=0)
            >>> stmt = Parser.parse_next_statement_static(location)
        """
        # Check if we've reached the end of input
        if location.byte_position >= len(location.input):
            return None

        service = ZetaSqlLocalService.get_instance()
        response = service.parse(
            sql_statement=location.input,
            parse_resume_location=location,
            options=options,
        )

        if hasattr(response, "resume_byte_position") and response.resume_byte_position is not None:
            location.byte_position = response.resume_byte_position

        return response.parsed_statement if response.parsed_statement else None

    @staticmethod
    def parse_next_script_statement_static(
        location: zetasql.types.ParseResumeLocation,
        options: zetasql.types.LanguageOptions | None = None,
    ) -> zetasql.types.ASTStatement | None:
        """Static method for parsing next script statement.

        Args:
            location: ParseResumeLocation that tracks position.
                     Its byte_position will be updated.
            options: Optional language options

        Returns:
            Parsed statement AST, or None if no more statements

        Example:
            >>> location = ParseResumeLocation(input=script, byte_position=0)
            >>> stmt = Parser.parse_next_script_statement_static(location)
        """
        # Check if we've reached the end of input
        if location.byte_position >= len(location.input):
            return None

        service = ZetaSqlLocalService.get_instance()
        response = service.parse(
            sql_statement=location.input,
            parse_resume_location=location,
            options=options,
            allow_script=True,
        )

        if hasattr(response, "resume_byte_position") and response.resume_byte_position is not None:
            location.byte_position = response.resume_byte_position

        return response.parsed_statement if response.parsed_statement else None

    @staticmethod
    def iterate_statements(
        script: str,
        options: zetasql.types.LanguageOptions | None = None,
    ):
        """Iterate through statements in a script, yielding parsed ASTs.

        This is a generator that lazily parses each statement in the script.
        Similar to Java's Iterator pattern.

        Args:
            script: SQL script with multiple statements
            options: Optional language options

        Yields:
            ASTStatement for each statement in the script

        Raises:
            ZetaSQLError: If parsing fails for any statement

        Example:
            >>> script = "SELECT 1; SELECT 2; SELECT 3;"
            >>> for stmt in Parser.iterate_statements(script):
            ...     print(type(stmt).__name__)
        """
        location = zetasql.types.ParseResumeLocation(input=script, byte_position=0)

        while location.byte_position < len(script):
            stmt = Parser.parse_next_statement_static(location, options)

            if stmt is None:
                break

            yield stmt



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: zetasql/public/simple_token_list.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'zetasql/public/simple_token_list.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&zetasql/public/simple_token_list.proto\x12\x07zetasql\"I\n\x14SimpleTokenListProto\x12+\n\ntext_token\x18\x02 \x03(\x0b\x32\x17.zetasql.TextTokenProtoJ\x04\x08\x01\x10\x02\"[\n\x0eTextTokenProto\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x11\n\tattribute\x18\x02 \x01(\x04\x12(\n\x0bindex_token\x18\x03 \x03(\x0b\x32\x13.zetasql.TokenProto\"-\n\nTokenProto\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x11\n\tattribute\x18\x02 \x01(\x04\x42+\n\x12\x63om.google.zetasqlB\x15SimpleTokenListProtos')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zetasql.public.simple_token_list_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022com.google.zetasqlB\025SimpleTokenListProtos'
  _globals['_SIMPLETOKENLISTPROTO']._serialized_start=51
  _globals['_SIMPLETOKENLISTPROTO']._serialized_end=124
  _globals['_TEXTTOKENPROTO']._serialized_start=126
  _globals['_TEXTTOKENPROTO']._serialized_end=217
  _globals['_TOKENPROTO']._serialized_start=219
  _globals['_TOKENPROTO']._serialized_end=264
# @@protoc_insertion_point(module_scope)
